import pickle as pkl
import numpy as np

applying_filter = True
#variables_to_test = ["tos", "sos", "vos", "uos", "zos"]
variables_to_test = ["bgheatco", "bgsaltco", "bgvole3t"]


def load_limits(filename="limits.pkl"):
    return pkl.load(open(filename, "rb"))


def load_rmse_file(_filename):
    # Routine that reads a netcdf file
    return pkl.load(open(_filename, "rb"))


def read_arguments():
    # Using optparse library to get command line options and arguments
    from optparse import OptionParser

    # Reading the command line arguments and
    # Creating the parser
    parser = OptionParser()
    # Adding options
    # Option for generating simplified output
    # parser.add_option("-r", help="Reference File", dest="reference", type="string", default="year_01")
    parser.add_option("-f", help="File to evaluate", dest="to_evaluate", type="string")

    # Parse arguments and options
    (options, args) = parser.parse_args()

    _evaluated_filename_pattern = options.to_evaluate

    return _evaluated_filename_pattern


def evaluate_dictionaries(simulation_rmse, limit_file):
    failed_variables = list_of_failed_variables(simulation_rmse, limit_file)

    if len(failed_variables) == 0:
        return True
    else:
        return False


def evaluate_fitness_from_dictionaries(simulation_rmse, limit_file):
    fitness = 0.
    for grid in simulation_rmse.keys():
        for variable in simulation_rmse[grid].keys():
            if applying_filter:
                if variable not in variables_to_test:
                    continue
            try:
                to_evaluate = simulation_rmse[grid][variable]
                steps = len(to_evaluate)
                ensemble_mean = limit_file[grid][variable][2][:steps]
                distance_from_ensemble_mean = to_evaluate - ensemble_mean
                # TODO: Improve this normalization, right now the successful last run had a fitness value > 300
                var_fitness = np.linalg.norm(distance_from_ensemble_mean)
                normalization_factor = np.average(ensemble_mean)
                if normalization_factor == 0:
                    normalization_factor = 1
                fitness += var_fitness / normalization_factor
            except KeyError:
                pass
    return fitness


def list_of_failed_variables(simulation_rmse, limit_file):
    # For each grid (In this case probably only grid_T)
    variables = []
    failed_variables = []
    for grid in simulation_rmse.keys():
        for variable in simulation_rmse[grid].keys():
            if applying_filter:
                if variable not in variables_to_test:
                    continue
            try:
                to_evaluate = simulation_rmse[grid][variable]
                steps = len(to_evaluate)
                reference_max = limit_file[grid][variable][1]
                reference_min = limit_file[grid][variable][0]
                # Use arrays of the same size
                reference_max = reference_max[:steps]
                reference_min = reference_min[:steps]

                # Check if the values exceed the maximum or doesn't reach the minimum
                more_than_maximum = (to_evaluate > reference_max).any()
                less_than_minimum = (to_evaluate < reference_min).any()
                failed = more_than_maximum or less_than_minimum
                variables.append(variable)
                if failed:
                    failed_variables.append(variable)
            except KeyError:
                pass
    return failed_variables