# Scipt that creates an HTML file to view a collection of pictures

from os import listdir
from os.path import join, basename


def command_line_arguments():
    # Using optparse library to get command line options and arguments
    from optparse import OptionParser
    # Creating the parser
    parser = OptionParser()
    # Adding options
    # Option for generating simplified output
    parser.add_option("-f", help="File to evaluate", dest="to_evaluate", type="string", default=None)

    # Parse arguments and options
    (options, args) = parser.parse_args()

    if options.to_evaluate is None:
        print("Argument -f folder is needed")
        exit(0)

    _data_folder = options.to_evaluate
    return _data_folder


def common_start(sa, sb):
    """ returns the longest common substring from the beginning of sa and sb """

    def _iter():
        for a, b in zip(sa, sb):
            if a == b:
                yield a
            else:
                return

    return ''.join(_iter())


def create_viewer(_data_folder, variable_summary=None):
    files = [_f for _f in listdir(_data_folder) if _f.count("png")]
    s0 = files[0]
    for f in files:
        s0 = common_start(s0, f)

    file_pattern = s0

    files = list(sorted(files))

    if variable_summary is not None:
        variable_header = """
        <div class="header">
          <h1>Variable Summary</h1>
    
    
          <p>%s</p>
          <a href="../">Go back<br></a>
          <b>Results</b>
        </div>
        """ % variable_summary.replace("\n", "<br>")

    head = '''
   <html>
   <head>
   <style>
   div.gallery {
     margin: 5px;
     border: 1px solid #ccc;
     float: left;
     width: 150px;
   }
   
   div.gallery:hover {
     border: 1px solid #777;
   }
   
   div.gallery img {
     width: 100%;
     height: auto;
   }
   
   div.desc {
     padding: 15px;
     text-align: center;
   }
   </style>
   </head>
   <body>
   '''

    if variable_summary:
        head = head.replace("<html>", "<html>" + variable_header)

    body_template = '''
   
   <div class="gallery">
     <a target="_blank" href="%s">
       <img src="%s" alt="" width="600" height="400">
     </a>
     
   </div>
   '''
#<div class="desc">%s</div>
    tail = '''
   </body>
   </html>
   '''

    def file_path(file_name):
        # return join(data_folder, file_name)
        return join(".", file_name)

    def var_name(file_name):
        return file_name.replace(file_pattern, "")

    body = "".join([body_template % (file_path(f), file_path(f)) for f in files])

    output_file = join(_data_folder, "viewer.html")
    with open(output_file, "w") as f:
        f.write(head + body + tail)
    print("File %s created!" % output_file)


if __name__ == "__main__":
    data_folder = command_line_arguments()
    create_viewer(data_folder)
