import numpy as np
import glob
# TODO: Probably should keep this module as a standalone module and avoid importing things from AutoRPE
from AutoRPE.EvaluationScripts.accuracy_test_with_limit_file import list_of_failed_variables

def read_arguments():
    # Using optparse library to get command line options and arguments
    from optparse import OptionParser

    # Reading the command line arguments and
    # Creating the parser
    parser = OptionParser()
    # Adding options
    # Option for generating simplified output
    # parser.add_option("-r", help="Reference File", dest="reference", type="string", default="year_01")
    parser.add_option("-f", help="File to evaluate", dest="to_evaluate", type="string")

    # Parse arguments and options
    (options, args) = parser.parse_args()
    print(options)

    _evaluated_filename_pattern = options.to_evaluate

    return _evaluated_filename_pattern


def load_pkl(_filename):
    import pickle as pkl

    return pkl.load(open(_filename, "rb"))


def plot_result(result=None, output_folder=None, limits=None,
                ensemble_members_path=None,
                variable_summary=None):
    import matplotlib
    import matplotlib.pyplot as plt

    matplotlib.use('Agg', force=True)
    # The result file has to be a dictionary containing the results from an experiment

    # Get member filenames
    members = []
    if ensemble_members_path is not None:
        filenames = glob.glob(ensemble_members_path)
        assert len(filenames) > 0
        # Load members data
        for filename in filenames:
            members.append(load_pkl(filename))

    # List of failed variables
    try:
        failed_variables = list_of_failed_variables(result, limits)
    except AttributeError:
        failed_variables = []



    if result is not None:
        grids = result.keys()
        variables = {}
        for grid in grids:
            variables[grid] = result[grid].keys()
    elif limits is not None:
        grids = limits.keys()
        variables = {}
        for grid in grids:
            variables[grid] = limits[grid].keys()
    elif len(members) > 0:
        # Pick one case to extract grid and variable names
        example_file = members[0]
        grids = example_file.keys()
        variables = {}
        for grid in grids:
            variables[grid] = example_file[grid].keys()
        del example_file
    else:
        raise AssertionError("No way of finding the grid and the variables")

    # grids = ["scalar"]
    # Start plotting
    for grid in grids:
        for variable in variables[grid]:
            if variable.count("time"):
                continue
            # Plot limits
            try:
                y0 = limits[grid][variable][0]
                y1 = limits[grid][variable][1]
                x0 = range(len(y0))
                plt.fill_between(x0, y0, y1, alpha=.2)
            except KeyError:
                print("No limits for %s in %s" % (variable, grid))

            if members:
                # Plot ensemble average and standard deviation
                var_data = np.array([m[grid][variable] for m in members])
                average = np.average(var_data, axis=0)
                std = np.std(var_data, axis=0)
                x0 = range(len(average))
                plt.fill_between(x0, average - std, average + std, color="green", alpha=.2)
                plt.plot(average, "--", color="green")

                # Plot each member
                for member in members:
                    plt.plot(member[grid][variable], color="gray", alpha=.2)

            # Plot result being evaluated
            if result is not None:
                vdata = result[grid][variable]
                plt.plot(vdata, color="red")
                plt.scatter(range(len(vdata)), vdata, color="red")

            plt.gcf().patch.set_facecolor('#e8bcbc')

            plt.title(variable, fontsize=50)

            x_loc = range(0, len(y0), 10)
            x_ticks = [5*_x for _x in x_loc]
            plt.xticks(x_loc, x_ticks)
            plt.xlabel("Time (days)")
            plt.ylabel("RMSE")
            if variable in failed_variables:
                plt.savefig("%s/%s.png" % (output_folder, variable), facecolor='#e8bcbc')
            else:
                plt.savefig("%s/%s.png" % (output_folder, variable))
            plt.clf()
    from AutoRPE.EvaluationScripts.gallery_creator import create_viewer
    create_viewer(output_folder, variable_summary=variable_summary)


if __name__ == "__main__":
    plot_result(result=None, output_folder="Figures")
