def create_explorer(experiment, vault):
    from os.path import abspath, join, dirname
    # Get all variables
    all_variable_ids = set(sum([j.analysis_set for j in experiment.all()], []))
    variables = [vault.get_variable_by_id(_id) for _id in all_variable_ids]

    # Take the failed experiments with a single variable
    failed_jobs_with_single_variable = [j for j in experiment.failed if len(j.analysis_set) == 1]
    failed_variable_ids = sum([j.analysis_set for j in failed_jobs_with_single_variable], [])
    # Other failed jobs
    failed_jobs_with_multiple_variables = [j for j in experiment.failed if len(j.analysis_set) > 1]
    # Sort jobs by amount of variables in the set
    failed_jobs_with_multiple_variables = list(sorted(failed_jobs_with_multiple_variables,
                                                      key=lambda x: len(x.analysis_set)))
    # Obtain the variables, and from them get also the modules and routines at which they belong
    failed_variables = [vault.get_variable_by_id(_id) for _id in failed_variable_ids]
    failed_modules = list(set([v.routine.module for v in failed_variables]))
    failed_routines = list(set([v.routine for v in failed_variables]))

    # Get all the modules and routines ( only those that are used during the execution)
    modules = list(set([v.routine.module for v in variables]))
    for m in modules:
        m.variables = [v for v in variables if v.routine.module == m]
        m.failed_variables = [v for v in failed_variables if v.routine.module == m]
    # Sorted by amount of failed variables
    modules = list(sorted(modules, key=lambda m: - len(m.failed_variables)))

    routines = list(set([v.routine for v in variables]))
    for r in routines:
        r.variables = [v for v in variables if v.routine == r]
    # Sorted by amount of failed variables
    routines = list(sorted(routines, key=lambda r: - len(r.variables)))

    # Start filling the template
    tree = ""
    tree += '<span class="box"><strong>Experiments with individual variables</strong>\n</span><ul class="nested">'
    # Loop through modules
    for module in modules:
        # Obtain module routines and variables
        mod_routines = [r for r in routines if r.module == module]
        mod_variables = [v for v in variables if v.routine.module == module]

        # Insert module html depending on it having failed variables
        if module in failed_modules:
            fraction_failed = len([v for v in failed_variables if v.routine.module == module])
            tree += '\t<li style="color:red;">' \
                    '<span class="box">%20s&emsp;%i/%i</span>\n\t\t<ul class="nested">\n' % (
            module.name, fraction_failed, len(mod_variables))
        else:
            continue
            tree += '\t<li style="color:green;"' \
                    '><span class="box">%20s&emsp;%i</span>\n\t\t<ul class="nested">\n' % (
            module.name, len(mod_variables))
        # Loop through routines
        for routine in mod_routines:
            # Obtain routine variables
            r_vars = [v for v in variables if v.routine == routine]
            # Insert routine html depending on it having failed variables
            if routine in failed_routines:
                fraction_failed = len([v for v in failed_variables if v.routine == routine])
                tree += '\t\t\t<li style="color:red;">' \
                        '<span class="box">%20s&emsp;%i/%i</span>\n\t\t\t\t<ul class="nested">\n' % (
                        routine.name, fraction_failed, len(r_vars))
            else:
                continue
                tree += '\t\t\t<li style="color:green;"><span class="box">%20s&emsp;%i</span>' \
                        '\n\t\t\t\t<ul class="nested">\n' % (
                        routine.name, len(r_vars))
            for var in r_vars:
                if var in failed_variables:
                    job = [j for j in failed_jobs_with_single_variable if var.id in j.analysis_set]
                    if len(job) > 1:
                        raise AssertionError()
                    else:
                        job = job[0]
                    wrong_outputs, plots_path = job.create_plots()
                    wrong_outputs = len(wrong_outputs)
                    viewer_path = join(plots_path, "viewer.html")
                    tree += '\t\t\t\t\t<li > <a href="%s" target="iframe_a">%s</a> %i</li>\n' \
                            % (viewer_path, var.name, wrong_outputs)

                else:
                    continue
                    tree += '\t\t\t\t\t<li style="color:green;"> %s</li>\n' % var.name
            tree += '\t\t\t\t</ul>\n\t\t\t</li>\n'
        tree += '\t\t</ul>\n\t</li>\n'

    # Add other jobs that failed
    tree += "</ul>"
    tree += '\t\t\t<li > \n <span class="box"><b> Other failed jobs </b></span>\n\t\t\t\t<ul class="nested">\n'
    for job in failed_jobs_with_multiple_variables:
        wrong_outputs, plots_path = job.create_plots()
        wrong_outputs = len(wrong_outputs)
        viewer_path = join(plots_path, "viewer.html")
        tree += '<li > <a href="%s" target="iframe_a">%s</a> %i</li>\n' \
                % (viewer_path, str(job.identifier()), wrong_outputs)
    tree += "</ul></li>"
    # Open template

    template_name = "explorer_template.html"
    template_path = join(dirname(__file__), template_name)
    with open(template_path, "r") as template:
        html = template.read()

    # Save test
    test_path = "test.html"
    with open(abspath(test_path), "w") as test:
        html = html % tree
        test.write(html)
        print(abspath(test_path))
