from AutoRPE.UtilsRPE.Functions import load_vault
import re
from os.path import dirname, join


def cml():
    # Getting Command line options
    from optparse import OptionParser

    parser = OptionParser()
    parser.add_option("-f", "--file", dest="file", default=None)
    parser.add_option("-v", "--vault", dest="vault", default=None)

    (options, args) = parser.parse_args()
    return options.file, options.vault


def variable_summary(variables):
    routines = [var.routine.name for var in variables]
    modules = [var.routine.module.name for var in variables]

    # Write a small report depending on how many different modules, routines and variables are present
    if len(set(modules)) > 1:
        text = "The namelist contains %i variables from %i modules:\n" % (len(variables), len(set(modules)))
        for mod in set(modules):
            text += "- Module %s: %i variables\n" % (mod, modules.count(mod))
    elif len(set(routines)) > 1:
        text = "The namelist contains %i variables from module %s.\n" % (len(variables), modules[0])
        text += "Variables are distributed in %i routines:\n" % len(set(routines))
        for routine in set(routines):
            text += "- Routine %s: %i variables\n" % (routine, routines.count(routine))
    elif len(variables) > 1:
        if routines[0] != "main":
            text = "The namelist contains %i variables from module %s and routine %s\n" % \
                   (len(variables), modules[0], routines[0])
        else:
            text = "The namelist contains %i variables from module %s.\n" % \
                   (len(variables), modules[0])
        text += "The variables are:\n"
        for v in variables:
            text += "- %s\n" % v.name
    elif len(variables) == 1:
        if routines[0] != "main":
            text = "The namelist contains only the variable %s from module %s and routine %s\n" \
                   % (variables[0].name, modules[0], routines[0])
        else:
            text = "The namelist contains only the variable %s from module %s.\n" \
                   % (variables[0].name, modules[0])
    else:
        text = "The namelist does not contain any variable"
    return text


def variables_from_namelist_precision(namelist_path, vault):
    # Load precision namelist
    with open(namelist_path) as f:
        data = [l for l in f]
    # Process namelist to obtain variables
    pattern = "emulator_variable_precisions\(([0-9]*)\)"
    variables = []
    for l in data:
        m = re.search(pattern, l, re.I)
        if m:
            var_id = int(m.group(1))
            var = vault.get_variable_by_id(var_id)
            variables.append(var)
    return variables


if __name__ == "__main__":
    namelist_path, vault_path = cml()
    # Load vault
    vault = load_vault(vault_path)

    variables = variables_from_namelist_precision(namelist_path, vault)

    summary = variable_summary(variables)

    print(summary)
    # Update viewer
    viewer_path = join(dirname(namelist_path), "viewer.html")

    with open(viewer_path, "r") as f:
        viewer = f.read()

    # Check if the viewer already contains the variable summary
    if viewer.count("Variable Summary"):
        exit(0)

    # It the summary its not present, create it and add it to the viewer
    header = """
    <div class="header">
      <h1>Variable Summary</h1>
      
    
      <p>%s</p>
      <a href="../">Go back<br></a>
      <b>Results</b>
    </div>
    """ % summary.replace("\n", "<br>")
    new_viewer = viewer.replace("<html>", "<html>" + header)

    with open(viewer_path, "w") as f:
        f.write(new_viewer)
