from AutoRPE.UtilsRPE.SourceManager import load_vault
from AutoRPE.UtilsRPE.Getter import get_type_of_contents
import re

# Few functions needed
def line_is_declaration(_line):
    _line = _line.strip()
    if not _line:
        return False
    first_character = _line[0]
    if first_character in ["!", "&"]:
        return False
    if _line.count("!"):
        _line = _line[:_line.find("!")]
    if _line.count("="):
        return True
    return False


def line_is_real_declaration(_line):
    if _line.count("!"):
        _line = _line[:_line.find("!")]
    value = _line.split("=")[1]
    if value.count(","):
        return False
    if value.count("'"):
        return False
    if not value.count("."):
        return False
    try:
        float(value)
        return True
    except ValueError:
        return False


def add_val_to_variable(_line):
    eq_index = _line.find("=")
    first_section = _line[:eq_index]
    second_section = _line[eq_index:]
    return "   %s%sval\t" % (first_section.strip(), "%") + second_section

if __name__ == "__main__":
    from optparse import OptionParser
    parser = OptionParser()
    parser.add_option("-v", "--vault", dest="vault", default=None, metavar="FILE",
                      help="Path to the vault database")
    parser.add_option("-o", "--output-namelist", dest="output", default="namelist_precisions", metavar="FILE",
                      help="Filename for the output namelist. Default=namelist_precisions")

    (options, args) = parser.parse_args()
    vault_path = options.vault
    output_filename = options.output

    vault = load_vault(vault_path)
    if vault is None:
        print("Not able to load %s !\n" % vault_path)
        exit(1)

    if args == []:
       print("namelists to adapt should be provided as arguments")
       exit(1)
    all_namelists = args
    
    pattern = "^([^\!]*)=([^\!]*)"
    
    for namelist in all_namelists:
        with open(namelist) as f:
            data = [line for line in f]
            for index, _l in enumerate(data):
                if line_is_declaration(_l):
                    m = re.search(pattern, _l)
                    var, value = m.groups()
                    type_of_var = get_type_of_contents(var, routine=None, vault=vault)
                    if type_of_var == "rpe_var":
                        var = "%s%%val" % var.strip()
                        data[index] = "%s = %s\n" % (var, value)
                    if type_of_var == "fld_n":
                        split_value = value.split(",")
                        value = value.replace(split_value[1], "-1, " + split_value[1])
                        data[index] = "%s = %s" % (var, value)
    
            text = "".join(data)
        with open(namelist, "w") as f:
            f.write(text)
