from AutoRPE.UtilsRPE.BasicStructures import Interface
from AutoRPE.UtilsRPE.SourceManager import load_variable_list, load_vault, expand_sources
from os.path import basename
import re


# Script that automatically creates different versions of a routine with different precisions
def fix_casts(source_path, vault_path, list_filename=None):
    vault = load_vault(vault_path)
    pattern = r"call.*\b(real\(.*)"
    cast_pattern = r"\b(real\(.*)"
    pattern_precision = r"\b([wds]p|[8]) *\)"
    call_pattern = r"call *\b(\w*)\b"

    lookup_table = {52: "dp", 23: "sp"}

    if list_filename is not None:
        # Update vault precisions with variable list
        variables_to_update = load_variable_list(list_filename)
        for variable in vault.variables:
            if variable.id in variables_to_update:
                variable.precision = 52

    file_paths = expand_sources(source_path=source_path)
    for file_path in file_paths:
        if not (file_path.count(".F90") or file_path.count(".f90") or file_path.count(".h90")):
            continue
        file_modified = False
        module_name = basename(file_path).split(".")[0]
        with open(file_path, "r", encoding="latin-1") as f:
            lines = [l for l in f]
            for line_index, line in enumerate(lines):
                # Check if there's a routine call with a cast to real
                if re.search(pattern, line, re.I):
                    try:
                        calls = calls_to_routines(line)
                    except ValueError:
                        continue
                    for call in calls:
                        arguments = find_call_arguments(call)
                        routine_name = re.search(call_pattern, call, re.I).group(1)
                        try:
                            routine = vault.get_procedure_by_name(routine_name)
                        except ProcedureNotFound:
                            continue
                        # If it is an interface, find the specific routine being used
                        if isinstance(routine, Interface):
                            try:
                                routine = match_arguments_and_interface(arguments, interface=routine, vault=vault)
                            except AssertionError:
                                routine = None
                        for index, argument in enumerate(arguments):
                            if re.search(cast_pattern, argument, re.I):
                                if routine is not None:
                                    if not is_keyword_argument(argument):
                                        target_precision = routine.get_variable_by_position(index).precision
                                    else:
                                        key = argument.split("=")[0].strip()
                                        target_precision = routine.get_variable_by_name(key).precision
                                else:
                                    # For now we will assume that in this case it is not necessary to change the cast
                                    continue
                                    # Problematic to assume that the precision of an external routine is either double or single
                                    # target_precision = 52
                                    target_precision = 23
                                try:
                                    target_precision = lookup_table[target_precision]
                                except KeyError:
                                    raise KeyError("target_precision %s not found in lookup_table", target_precision)
                                cast = close_brackets(re.search(cast_pattern, argument, re.I).group(1))
                                precision = re.search(pattern_precision, cast, re.I).group(1)
                                if precision.strip() == target_precision:
                                    continue
                                else:
                                    cast_replacement = re.sub(r"(.*\b)[wds]p( *\).*)", r"\1%s\2" % target_precision,
                                                              cast, re.I)
                                    line = line.replace(cast, cast_replacement)
                                    lines[line_index] = line
                                    file_modified = True
        if file_modified:
            text = "".join(lines).strip()
            with open(file_path, "w") as f:
                f.write(text)


if __name__ == "__main__":
    # Getting Command line options
    from optparse import OptionParser

    help_text = """
    """
    parser = OptionParser(usage=help_text)
    parser.add_option("-l", "--variable-list", dest="variable_list",
                      help="Path to the filename with the list of variables.", default=None, metavar="FILE")
    parser.add_option("-v", "--vault", dest="vault", default=None, metavar="FILE")
    parser.add_option("-s", "--input-source", dest="input_source", default=None, metavar="FILE")

    (options, args) = parser.parse_args()
    vault_path = options.vault
    source_path = options.input_source
    list_filename = options.variable_list

    if source_path is None or vault_path is None:
        print("\nArgument -s/--input-source and -v/--vault are mandatory\n")
        parser.print_help()
        exit(1)
    fix_casts(source_path, vault_path, list_filename)
