import AutoRPE.UtilsRPE.VariablePrecision as VariablePrecision
from AutoRPE.UtilsRPE.Functions import remove_comments, load_vault, \
                                       replace_variable_exact_match, find_call_arguments, \
                                       calls_to_routines, calls_to_functions
from AutoRPE.UtilsRPE.ImplementRPE import ImplementRPE
from AutoRPE.UtilsRPE.CurrentBlock import CurrentBlock
from AutoRPE.UtilsRPE.Cleaner import expand_source, merge_line_range
from AutoRPE.UtilsRPE.TrackDependency import retrieve_real_in_line
from os.path import basename
import warnings
import re


def remove_keywords(arguments):
    patt = "[a-zA-Z_0-9]*=(.*)"
    for index, argument in enumerate(arguments):
        m = re.match(patt, argument)
        if m:
            arguments[index] = m.group(1)
    return arguments


def find_call_in_original_file(routine, argument, argument_index, _file, target_type="dp", fix=False):
    # Add a cast to the proper type
    new_argument = "REAL(%s, %s)" % (argument, target_type)
    # Fix in case the argument already had a cast, in which case we'll only change the type of the cast
    if new_argument.count("REAL(REAL("):
        patt = "REAL\((.*),(.*)\)"
        m = re.search(patt, argument)
        new_argument = "REAL(%s, %s)" % (m.group(1), target_type)
    # Fix the pattern to account for regex rules
    pattern = r"\b%s\b####\b%s" % (routine, argument)
    pattern = pattern.replace("(", "\(")
    pattern = pattern.replace(")", "\)")
    pattern = pattern.replace("+", "\+")
    pattern = pattern.replace("*", "\*")
    pattern = pattern.replace("####", ".*")
    pattern += r"(\b|[, )])"

    # Initialize found to False
    found = False

    # Open file
    with open(_file, "r") as f:
        lines = [l for l in f]
        for line_index, line in enumerate(lines):
            if line.lower().count("subroutine") or line.lower().count("function"):
                continue

            # In case there's a call to the target routine with a line continuation, fix it and call again the function
            if line.lower().count(routine):
                # The line contains the routine name but the pattern doesn't match
                line = remove_comments(lines[line_index]).strip()
                # Check if it is a call with split args
                if line and line.strip()[-1] == "&":
                    line = merge_line_range(_file, line_index, fix_in_file=False)
                    m = re.search(pattern, line, re.I)
                    if m:
                        # It merge the split call in one single line
                        merge_line_range(_file, line_index, fix_in_file=True)
                        # Now the function can properly fix the call
                        return find_call_in_original_file(routine, argument, argument_index, _file, target_type, fix)
            else:
                continue

            # Look for the call to the routine with the proper arguments
            m = re.search(pattern, line, re.I)
            if m:
                # The args have already been fixed

                # TODO: Check whether this is still necessary
                # if line.count(new_argument) > 0:
                #    continue
                calls = calls_to_routines(line)
                if len(calls) == 0:
                  calls = calls_to_functions(line, vault=vault)

                if len(calls) > 1:
                    print("Now what?")
                arguments = remove_keywords(find_call_arguments(calls[0]))
                # Check if the argument is placed in the proper place
                try:
                    if arguments.index(argument) != argument_index:
                        continue
                except ValueError:
                    # It seems that this can happen if the exact same combination
                    # of routine argument has been fixed before.
                    continue
                found = True
                # TODO: Need to account for the possibility of having the same argument in different positions.
                new_line = replace_variable_exact_match(argument, new_argument, line)
                lines[line_index] = new_line+"\n"

    # In case we found the call overwrite the file with the fix
    # Otherwise return False
    if found:
        if fix:
            code = "".join(lines).strip()
            with open(_file, "w") as output:
                output.write(code)
        return True
    else:
        return False


def code_check_file(filename, vault, fix=False, original_sources=None):
    try:
        current_block = CurrentBlock(vault[filename.split("/")[-1].replace(".f90", "")].main())
    except AttributeError:
        warnings.warn("No module %s" % filename)
        return
    f = ImplementRPE(filename, None)
    if original_sources:
        original_filename = original_sources[filename]

    f.pre_process()
    # Loop through lines
    for line in f.lines:
        # Returns all REAL var on the line which corresponding dummy argument has different type
        procedure_tf, arguments_proc_tf, argument_index_tf, dummy_var_type_tf, dummy_var_intent_tf = \
            get_real_in_line(line, vault, current_block=current_block, check=True)
        # If something was found check if can be fixed
        if [procedure_tf, arguments_proc_tf, argument_index_tf, dummy_var_type_tf] != [None, None, None, None]:
            for p_tf, ap_tf, ai_tf, dv_tf, dv_i_tf in zip(procedure_tf, arguments_proc_tf, argument_index_tf,
                                                          dummy_var_type_tf, dummy_var_intent_tf):
                types = {"single": "sp", "double": "dp"}
                try:
                    target_type = types[dv_tf]
                except KeyError:
                    continue
                # If the intent is inout or out it can not be fixed with a cast
                if fix:
                    if dv_i_tf.count("out"):
                        fix_this = False
                    else:
                        fix_this = True
                else:
                    fix_this = False

                fixed = find_call_in_original_file(p_tf, ap_tf, ai_tf, filename,
                                                   target_type=target_type, fix=fix_this)
                if original_sources:
                    original_fixed = find_call_in_original_file(p_tf, ap_tf, ai_tf, original_filename,
                                                                target_type=target_type, fix=fix_this)


                if fixed:
                    if fix:
                        print("Was fixed!")
                    else:
                        print("Can be fixed!")
                else:
                    print("Can't be fixed")

        else:
            # If not, look if the code is entering or exiting a routine block
            current_block = current_block.in_which_block(line, vault=vault)


if __name__ == "__main__":
    # Getting Command line options
    from optparse import OptionParser

    parser = OptionParser()
    parser.add_option("-v", "--vault", dest="vault", default=None, metavar="FILE")
    parser.add_option("-s", "--input-source", dest="input_source", default=None, metavar="FILE")
    parser.add_option("--additional-source", dest="additional_source", default=None, metavar="FILE")
    parser.add_option("-w", "--working-precision", dest="working_precision", default="single")
    parser.add_option("-t", "--target-precision", dest="target_precision", default="double")
    parser.add_option("-f", "--fix-dependencies", dest="fix_dependencies", default=False, action="store_true")

    (options, args) = parser.parse_args()
    vault_path = options.vault
    VariablePrecision.wp = options.working_precision
    target_precision = options.target_precision
    path_to_input_sources = options.input_source
    additional_sources_path = options.additional_source
    fix_dependencies = options.fix_dependencies

    if path_to_input_sources is None or vault_path is None:
        print("\nArguments -s/--input-source and -v/--vault are mandatory\n")
        parser.print_help()
        exit(1)

    # Loading vault
    vault = load_vault(vault_path)

    # Find all files in sources
    all_sources = expand_sources(path_to_input_sources)

    if additional_sources_path is not None:
        additional_sources = expand_sources(additional_sources_path)

        def match_sources(preprocessed, original):
            dictionary = {}
            for file in preprocessed:
                F90_filename = basename(file).replace(".f90", ".F90")
                for f in original:
                    if basename(f) == F90_filename:
                        dictionary[file] = f
                        break
            return dictionary
        dictionary_of_additional_sources = match_sources(all_sources, additional_sources)
    else:
        dictionary_of_additional_sources = None

    if not all_sources:
        parser.print_help()
        print("\nSources folder provided does not contain source files.\n")
        exit(1)
    # Process Sources!
    total = len(all_sources)
    for index, filename in enumerate(all_sources):
        code_check_file(filename, vault, fix=fix_dependencies, original_sources=dictionary_of_additional_sources)
