from AutoRPE.UtilsRPE.CodeTools import expand_sources
from AutoRPE.UtilsRPE.BasicStructures import Routine, Interface
from AutoRPE.UtilsRPE.ImplementRPE import ImplementRPE
from AutoRPE.UtilsRPE.CurrentBlock import CurrentBlock
from AutoRPE.UtilsRPE.Functions import load_variable_list, load_vault, calls_to_routines, find_called_routine, find_call_arguments, retrieve_variable, store_variable_list
from AutoRPE.UtilsRPE.Error import ProcedureNotFound
from os.path import basename
import re


real_types = ["single","double"]
def get_command_line_arguments():
   from optparse import OptionParser
   from os.path import isfile, isdir
   # Parse and assert command line options
   parser = OptionParser()
   parser.add_option("-s", "--source", dest="source_path",
                     help="Path to the source that will be modified.", default=None)
   parser.add_option("-v", "--vault", dest="vault_path",
                     help="Path to the vault.", default=None, metavar="FILE")
   parser.add_option("-o", "--output", dest="output",
                     help="Output filename", default=None, metavar="FILE")
   
   (options, args) = parser.parse_args()
   
   source_path = options.source_path
   vault_path = options.vault_path
   output_filename = options.output
   try:
   	assert source_path is not None, "Argument -s/--source required!"
   	assert vault_path is not None, "Argument -v/--vault required!"
   	assert isdir(source_path), "Argument -s/--source  %s is not a valid directory." % source_path
   	assert isfile(vault_path), "Argument -v/--vault %s is not a file." % vault_path
   except AssertionError as error:
        parser.print_help()	
        print("\n\n")
        print(error)
        exit(1)
   
   return source_path, vault_path, output_filename

source_path, vault_path, output_filename= get_command_line_arguments()


exceptions = ["mpi_", "xios_", "agrif_", "cice","ice_","wheneq","trc_","flush"]

vault = load_vault(vault_path)

files = expand_sources(source_path)


external_routines_found = []
variables = []
for filepath in files:
   module_name = basename(filepath).split(".")[0]
   module = vault[module_name]
   if module is None:
      continue
   file_info = ImplementRPE(filepath, vault)
   file_info.pre_process()

   current_block = CurrentBlock(module.main())
   for index, line in enumerate(file_info.lines):
      print("\r %04i/%04i %50s" % (index+1, len(file_info.lines), module_name),end="")
      try:
         routine_calls = calls_to_routines(line)
      except ValueError:
         continue
      if routine_calls:
         for call in routine_calls:
            is_external = False
            called_routine = find_called_routine(call)

            # Find the procedure called (object)
            try:
               procedure = vault.get_procedure_by_name(called_routine)
            except ProcedureNotFound:
               procedure = None
               pass

            if isinstance(procedure, Routine):
                continue
            elif isinstance(procedure, Interface):
                continue
            else:
                is_external = True
            if is_external:
               external_routines_found.append(called_routine)
               # Starting with the exceptions!"
               is_exception = False
               for exception in exceptions:
                  if called_routine.count(exception):
                     is_exception = True
                     break
               if is_exception:
                  continue
               called_arguments = find_call_arguments(call)
               if not called_arguments:
                   # If there's no argument, continue
                   continue
               else:
                   for argument in called_arguments:
                      try:
                         variable = retrieve_variable(argument, routine=current_block.routine, vault=vault)
                      except AssertionError:
                         continue
                      except ValueError:
                         continue
                      if variable is not None:
                         if variable.type in real_types:
                            variables.append(variable.id)
      else:
         current_block = current_block.in_which_block(line, vault=vault)
print()
variables = list(set(variables))
store_variable_list(variables, output_filename, vault)

