# Import classes and functions from RPE_Utils
from AutoRPE.UtilsRPE.MainFunctions import parse_sources, postprocess_sources, assign_sbits_to_rpe_variables, preprocess_sources
from AutoRPE.UtilsRPE.Functions import load_vault, load_list_of_files_to_keep_unmodified
from os import listdir, mkdir, rename
from os.path import join, isdir
from shutil import rmtree

if __name__ == "__main__":

    from optparse import OptionParser
    parser = OptionParser()
    parser.add_option("-i", "--input-source", dest="input_source", default=None, metavar="FILE", help="Path to a folder that contains the preprocessed code source")
    parser.add_option("-o", "--output-vault", dest="output_vault", default="vault.pkl", help="Filename of the output database. Default=vault.pkl")

    (options, args) = parser.parse_args()
    path_to_input_sources = options.input_source
    output_vault=options.output_vault
    
    if path_to_input_sources is None:
       print("\n\nArgument -s/--input-source is required\n\n")
       parser.print_help()
       exit(1)

    # Paths to source folders
    #path_to_input_sources = "./ModelSources/Input/"
    path_to_preprocessed_sources = "./tmp_processedsources/"
    path_to_output_sources = "./tmp_outputsources/"

    if not isdir(path_to_preprocessed_sources):
       mkdir(path_to_preprocessed_sources)
    if not isdir(path_to_output_sources):
       mkdir(path_to_output_sources)

#   path_to_preprocessed_sources = "./ModelSources/Processed/"
#   path_to_output_sources = "./ModelSources/Output/"

    print("Starting the implementation of the reduced precision emulator in:\n   %s\n\n" % path_to_input_sources)

    # List of files that should not be processed, as red from "list_of_files_to_keep_unmodified.txt"
    preprocess_blacklist = load_list_of_files_to_keep_unmodified()

    # Format text + replace real declarations with type(rpe_var)
    preprocess_sources(path_to_input_sources, path_to_preprocessed_sources, blacklist=preprocess_blacklist)

    # Obtain source information
    vault = parse_sources(path_to_preprocessed_sources, save_vault=True)

    # We load the code information into the vault
    try:
        vault
    except NameError:
        vault = load_vault()

    # Process the sources to fix call to functions, read/write, ...
    postprocess_sources(path_to_preprocessed_sources, path_to_output_sources, vault=vault)

    # Insert precision specification
    add_sbits_to_rpe_variables(path_to_output_sources, vault=vault)

    # Assign ids to variables that still don't have any (can't have it's precision changed online)
    # and set them as mutable=False
    vault.assign_ids_to_immutable_variables()

    # Add the read_precisions module into a file (should be accessible for all the other modules)
    file_to_add_read_precisions_module = None
    insert_read_precisions_module_to_file(path_to_output_sources, file_to_add_read_precisions_module,
                                                  vault=vault)

        # Deleting temporary folders
    if isdir(path_to_preprocessed_sources):
       rmtree(path_to_preprocessed_sources)

    output_folder = "SourcesWithRPE"
    if isdir(output_folder):
       rmtree(output_folder)
    if isdir(path_to_output_sources):
       rename(path_to_output_sources, output_folder)

