def get_command_line_arguments():
    from optparse import OptionParser
    from os.path import isfile, isdir
    # Parse and assert command line options
    parser = OptionParser()
    parser.add_option("-l", "--variable-list", dest="variable_list",
                      help="Path to the filename with the list of variables.", default=None, metavar="FILE")
    parser.add_option("-t", "--target-precision", dest="target_precision",
                      help="Precision specification at which the variables in the list will be set", default="dp")
    parser.add_option("-w", "--working-precision", dest="working_precision",
                      help="Precision used for wp variables in the code. i.e. double for standard runs,"
                           "single for mixed-precision versions", default="single")

    parser.add_option("-s", "--source", dest="source_path",
                      help="Path to the source that will be modified.", default=None)
    parser.add_option("-v", "--vault", dest="vault_path",
                      help="Path to the vault.", default=None, metavar="FILE")

    (options, args) = parser.parse_args()

    list_filename = options.variable_list
    source_path = options.source_path
    vault_path = options.vault_path
    target_precision = options.target_precision
    working_precision = options.working_precision
    try:
        assert list_filename is not None, "Argument -l/--variable-list required!"
        assert source_path is not None, "Argument -s/--source required!"
        assert vault_path is not None, "Argument -v/--vault required!"
        assert isfile(list_filename), "Argument -l/--variable-list %s is not a file." % list_filename
        assert isdir(source_path), "Argument -s/--source  %s is not a valid directory." % source_path
        assert isfile(vault_path), "Argument -v/--vault %s is not a file." % vault_path
    except AssertionError as error:
        parser.print_help()
        print("\n\n")
        print(error)
        exit(1)

    return list_filename, source_path, vault_path, target_precision, working_precision


# Reading the command line arguments
list_filename, source_path, vault_path, target_precision, working_precision = get_command_line_arguments()

# Setting the working precision
import AutoRPE.UtilsRPE.VariablePrecision as VariablePrecision
VariablePrecision.wp = working_precision

# Calling to the change_code function
from AutoRPE.UtilsRPE.CodeTools import change_code
change_code(list_filename, source_path, vault_path, target_precision)
