from AutoRPE.UtilsRPE.CodeTools import expand_sources
from AutoRPE.UtilsRPE.ObtainSourceFileInfo import ReadSourceFile
from AutoRPE.UtilsRPE.Functions import remove_comments, find_call_arguments, close_brackets

import re


def get_command_line_arguments():
    from optparse import OptionParser
    from os.path import isdir
    # Parse and assert command line options
    parser = OptionParser()
    parser.add_option("-s", "--source", dest="source_path",
                      help="Path to the source that will be modified.", default=None)

    (options, args) = parser.parse_args()

    source_path = options.source_path
    try:
        assert source_path is not None, "Argument -s/--source required!"
        assert isdir(source_path), "Argument -s/--source  %s is not a valid directory." % source_path
    except AssertionError as error:
        parser.print_help()
        print("\n\n")
        print(error)
        exit(1)

    return source_path


def check_completion(string):
    try:
        close_brackets(string)
    except ValueError:
        return False
    return True


def get_complete_call(lines, start):
    line = remove_comments(lines[start]).strip()
    _index = start+1
    while not check_completion(line):
        line += remove_comments(lines[_index]).strip()
        _index += 1
    line = line.replace("&&", "")
    line = line.replace("  ", "")
    return line, _index - start - 1


pattern = "CALL *lbc_lnk_multi"

source_path = get_command_line_arguments()


file_paths = expand_sources(source_path=source_path)

for filepath in file_paths:
    f = ReadSourceFile(filepath)
    something_changed = False
    for index, line in enumerate(f.lines):
        line = remove_comments(line).strip()
        match = re.search(pattern, line, re.IGNORECASE)
        if match:
            if line[-1] == "&":
                line, lines_to_remove = get_complete_call(f.lines, index)
                for i in range(lines_to_remove):
                    f.lines[index+1+i] = ""
            arguments = find_call_arguments(remove_comments(line))

            if ((len(arguments)-1) % 3) == 0:
                number_of_calls = int((len(arguments)-1) / 3)
                call_template = "CALL lbc_lnk( %s, %%s, %%s, %%s)" % arguments[0]
            else:  # If this happens there's a keyword argument at the end
                number_of_calls = int((len(arguments) - 2) / 3)
                call_template = "CALL lbc_lnk( %s, %%s, %%s, %%s, %s)" % (arguments[0], arguments[-1])

            substitutions = ""
            for i in range(number_of_calls):
                substitutions += call_template % tuple(arguments[i*3+1:(i+1)*3+1]) + "\n"
            substitutions = substitutions.strip()
            f.lines[index] = substitutions
            something_changed = True
    if something_changed:
        f.rebuild_text()
        with open(filepath, "w") as output:
            output.write(f.text)

