# This is an example of how to use the Precision Analysis Class
from PrecisionAnalysis import PrecisionAnalysis
from UtilsRPE.Functions import load_vault

# These are the parameters that are needed to launch a precision analysis experiment

experiment_name = "NEMO4-PrecisionAnalysis"


# Hostname of the remote machine that will be used to launch the simulations
remote_hostname = "mn3.bsc.es"

# Username used in the remote machine
remote_user = "pr1egi23"

# Path to the folder where the simulations will be launched
remote_scratch = "/gpfs/scratch/pr1egi00/pr1egi23/BA_NEMO"

# Path to the simulation template.
job_template = "SimulationTemplates/NEMOsim.sh"
# Path to the assertion template ("" if not using any extra assertion)
# In our case we are launching the same experiment with different initial conditions.
assertion_template = "SimulationTemplates/NEMOassertion.sh"

# Path to the vault
vault_path = "vault.pkl"


vault = load_vault(vault_path)

# Dictionary containing the variables and the values of the thresholds that we don't want our final set to overcome.
error_thresholds = thresholds = {"thetao": 1.49e-06, "so": 6.41e-06, "heatc": 2.74e-06, "saltc": 5.47e-09}

# Target reduced precision level
reduced_precision_level = 10

# Maximum number of simultaneously running jobs
max_running_jobs = 10

# In case restarting a experiment from a checkpoint
restarting_from_checkpoint = True

# Create experiment object
experiment = PrecisionAnalysis(remote_hostname=remote_hostname, remote_user=remote_user, remote_scratch=remote_scratch,
                               job_template=job_template, vault=vault, error_thresholds=error_thresholds,
                               reduced_precision_level=reduced_precision_level, max_running_jobs=max_running_jobs,
                               experiment_name=experiment_name, restarting_from_checkpoint=restarting_from_checkpoint,
                               assertion_template=assertion_template)

# Launch experiment
experiment.run()

# The experiment will create a file named EXPERIMENT_NAME_final_set.pkl
