from AutoRPE.UtilsRPE.MainFunctions import parse_sources, preprocess_sources
from AutoRPE.UtilsRPE.Functions import load_vault
from AutoRPE.UtilsRPE.TrackDependency import track_dependencies, propagate_dependencies
from os import listdir, mkdir
from os.path import isdir
from shutil import rmtree
import pickle
import sys


if __name__ == "__main__":
    # Getting Command line options
    from optparse import OptionParser
    parser = OptionParser()
    parser.add_option("-v", "--vault", dest="vault", default=None, metavar="FILE")
    parser.add_option("-s", "--input-source", dest="input_source", default="../ModelSources/Input", metavar="FILE")
    parser.add_option("-p", "--preprocessed-source", dest="path_to_preprocessed_sources", default="../ModelSources/Processed", metavar="FILE")

    (options, args) = parser.parse_args()
    vault_path = options.vault
    path_to_input_sources = options.input_source
    path_to_preprocessed_sources = options.path_to_preprocessed_sources

    if path_to_input_sources is None and vault_path is None:
       print("\nOne of the arguments -s/--input-source or -v/--vault is mandatory\n")
       parser.print_help()
       exit(1)

    print("Creating a database from sources from path_to_preprocessed_sourcesd in:\n   %s\n\n" % path_to_input_sources)
    
    # In case no vault has been provided it will create it from the sources in the input sources folder
    if vault_path is None:

        path_to_preprocessed_sources = "./tmp_processedsources/"
        if not isdir(path_to_preprocessed_sources):
            mkdir(path_to_preprocessed_sources)
        # List of files that should not be processed, as red from "list_of_files_to_keep_unmodified.txt"
        preprocess_blacklist = listdir(path_to_input_sources)

        # Format text + replace real declarations with type(rpe_var)
        preprocess_sources(path_to_input_sources, path_to_preprocessed_sources, blacklist=preprocess_blacklist)

        # Obtain source information
        vault = parse_sources(path_to_preprocessed_sources, save_vault=True)

        # Deleting temporary folders
        if isdir(path_to_preprocessed_sources):
            rmtree(path_to_preprocessed_sources)

    else:
        vault = load_vault(vault_path)

    try:
        # Check if variables already have the property
        same_as = vault.variables[0].same_as
    except AttributeError:
        # In case they don't add it
        for var in vault.variables:
            var.same_as = []

    # Fill the list of equivalent variables
    track_dependencies(path_to_preprocessed_sources, vault)
    # Create groups
    propagate_dependencies(vault)


    sys.setrecursionlimit(9000)
    pickle.dump(vault, open("vault_same_as.pkl", "wb"))
