from AutoRPE.UtilsRPE.Functions import load_vault
from AutoRPE.UtilsRPE.BasicStructures import Interface

# Getting Command line options
from optparse import OptionParser
parser = OptionParser()
parser.add_option("-v", "--vault", dest="vault", default=None, metavar="FILE")
parser.add_option("-n","--name",  dest="var_name", default=None)
parser.add_option("--name-contains",  dest="var_name_contains", default=None)
parser.add_option("-r","--routine",  dest="routine_name", default=None)
parser.add_option("-i","--interface",  dest="interface_name", default=None)
parser.add_option("-m","--module",  dest="module_name", default=None)
parser.add_option("-e","--expand",  dest="expand", default=False,action="store_true")
parser.add_option("--id",  dest="var_id", default=None )
(options, args) = parser.parse_args()


vault_path = options.vault
var_name = options.var_name
var_name_contains = options.var_name_contains
routine_name = options.routine_name
module_name = options.module_name
interface_name = options.interface_name
expand = options.expand
var_id = options.var_id

# Loading vault (source database)
if vault_path is None:
   print("\n\nArgument -v/--vault is required\n")
   parser.print_help()
   exit(1)
vault = load_vault(vault_path)


# Load all the variables in the vault and start filtering by the command line parameters provided.
#variables = [v for v in vault.variables if v.type in ["real", "rpe_var"]]
variables = vault.variables
# Sort by id
#variables = list(sorted(variables, key=lambda x: x.id))

# Filter by var id
if var_id is not None:
   var_id = int(var_id)
   variables = [ v for v in variables if v.id == var_id ]

# Filter by exact name coincidence
if var_name is not None:
   variables = [ v for v in variables if v.name == var_name ]

# Filter by name contains
if var_name_contains is not None:
   variables = [ v for v in variables if v.name.count(var_name_contains) ]
 
# Filter by routine name coincidence
if routine_name is not None:
   variables = [ v for v in variables if v.routine.name == routine_name ]

# Filter by module name coincidence
if module_name is not None:
   variables = [ v for v in variables if v.routine.module.name == module_name ]

# Filter by interface name
if interface_name is not None:
   interfaces = [ p for p in vault.procedures if isinstance(p, Interface)]
   try:
      my_interface = [ i for i in interfaces if i.name == interface_name][0]
   except IndexError:
      print("Inferface not found!")
      exit(0)
   routine_names = [p.name for p in my_interface.procedures]
   variables = [ v for v in variables if v.routine.name in routine_names ]
   

if len(variables) == 0:
   print("No results found")
   exit(0)

print("#%5s     %20s %25s %20s\n%s\n" %( "ID","variable","routine","module", 100*"#") )
line_format = "%5i    # %20s %25s %20s %10s, Same_as:%i"
for v in variables:
   if v.id is None:
      v.id = -1
   if v.same_as is None:
      v.same_as = []

   print( line_format % (v.id, v.name, v.routine.name, v.routine.module.name, v.type, len(v.same_as)) )
   if expand:
      for v2 in v.same_as:
         print("\t",v2.name)
