import AutoRPE.UtilsRPE.VariablePrecision as VariablePrecision


def literal_mask(string):
    splitted_string = [c for c in string]
    length = len(splitted_string)
    mask = []
    inside_literal = False
    marker = None
    for i in range(length):
        if splitted_string[i] == "\"":
            if not inside_literal:
                inside_literal = True
                marker = "\""
                mask.append(False)
            else:
                if marker == "\"":
                    inside_literal = False
                    marker = None
                    mask.append(False)
                elif marker == "'":
                    mask.append(True)
                else:
                    raise AssertionError("Should not happen!")
        elif splitted_string[i] == "'":
            if not inside_literal:
                inside_literal = True
                marker = "'"
                mask.append(False)
            else:
                if marker == "'":
                    inside_literal = False
                    marker = None
                    mask.append(False)
                elif marker == "\"":
                    mask.append(True)
                else:
                    raise AssertionError("Should not happen!")
        else:
            if not inside_literal:
                mask.append(False)
            else:
                mask.append(True)

    return mask


def remove_literals(string):
    mask = literal_mask(string)
    modified_string = "".join(
        [c if not mask[i] else " " for i, c in enumerate(string)])
    return modified_string


def close_brackets(string):
    started = False
    bracket_counter = 0
    mask = literal_mask(string)

    for _i, _c in enumerate(string):
        if _c == "(" and not mask[_i]:
            bracket_counter += 1
            if not started:
                started = True
        elif _c == ")" and not mask[_i]:
            bracket_counter -= 1
            if not started:
                raise ValueError("This should not happen, check arguments %s" % string)
        if bracket_counter < 0:
            raise ValueError("This should not happen, check arguments %s" % string)
        if started and bracket_counter == 0:
            return string[:_i + 1]

    if string.count("(") + string.count(")") == 0:
        return string
    raise ValueError("This should not happen, check arguments %s" % string)


def combine_types(list_of_types):
    to_return = list_of_types[0]
    for t in list_of_types:
        to_return = resulting_type(to_return, t)
    return to_return


def create_type_combinations(case_types):
    """
    Given a list of variable types, it creates all the possible combinations casting real types.
    :param case_types: list of variable types
    :return: list of all the possible variations of variable types
    """
    import itertools
    real_types = ["single", "double"]
    syn_types = ["real" if x in real_types else x for x in case_types]

    number_of_reals = syn_types.count("real")

    a = ["single", "double"] * number_of_reals
    combinations = list(set(itertools.permutations(a, number_of_reals)))

    new_types = []
    for comb in combinations:
        comb_types = syn_types[:]
        for i in range(number_of_reals):
            comb_types[comb_types.index("real")] = comb[i]
        new_types.append(comb_types)
    return new_types


def resulting_type(type_a, type_b):
    types = ["rpe_var", "complex", VariablePrecision.dp, VariablePrecision.sp, VariablePrecision.wp, "integer", "char", "logical"]
    for t in types:
        if type_a == t:
            return t
        if type_b == t:
            return t


def is_all_inside_parenthesis(string):
    if string.strip()[0] == "(":
        try:
            if "c%s" % string.strip() == close_brackets("c%s" % string.strip()):
                return True
            else:
                return False
        except ValueError:
            return False
    else:
        return False

