class Variable:
    def __init__(self, _routine, _name, _type, _dimension, _module=None):
        self.id = None
        self.module = _module
        self.routine = _routine
        self.name = _name
        self.type = _type
        if self.type == "real":
            print("A real has been declared: %s" % _name)
        self.dimension = _dimension
        self.is_dummy_argument = False
        self.intent = None
        self.is_optional = None
        self.is_retval = False
        self.pointer = None
        self.position = None
        self.allocatable = False
        self.precision = None
        self.is_parameter = None
        self.is_member_of = None
        self.mutable = None
        self.same_as = []

    def set_precision(self, precision):
        assert isinstance(precision, int), "Precision argument in set_precision must be an integer"
        self.precision = precision


class Procedure(object):
    def __init__(self, _name, module):
        self.name = _name
        self.module = module
        self.variables = []
        self.precision = None
        self.interface = None
        self.dummy_arguments = None
        self.uses = []
        self._accessible_variables = None

    def add_variable(self, _variable):
        try:
            assert isinstance(_variable, Variable)
        except AssertionError:
            pass
        _variable.routine = self
        self.variables.append(_variable)

    def get_variable_by_position(self, position):
        for variable in self.variables:
            if variable.is_dummy_argument:
                if variable.position == position:
                    return variable

    def get_variable_by_name(self, name):
        for variable in self.variables:
            if variable.name.lower() == name.lower():
                return variable

    def get_dummy_arguments(self):
        if self.dummy_arguments is None:
            number_of_dummy_arguments = len([True for var in self.variables if var.is_dummy_argument])
            self.dummy_arguments = [self.get_variable_by_position(i) for i in range(number_of_dummy_arguments)]
        return self.dummy_arguments


class Module:
    def __init__(self, _name):
        self.name = _name
        self.repo = None
        self.procedures = []
        self.variables = []
        self.precision = None
        self.uses = []
        self.used_by = []
        self.accessible_var = []
        self.indirect_accessible_var = []

    def add_procedure(self, _routine):
        assert isinstance(_routine, Procedure)
        _routine.module = self
        self.procedures.append(_routine)

    def add_variable(self, _variable):
        assert isinstance(_variable, Variable)
        _variable.routine = self
        self.variables.append(_variable)

    def __getitem__(self, item):
        for _routine in self.procedures:
            if _routine.name == item:
                return _routine
        new_routine = Routine(None, item)
        new_routine.module = self
        self.repo.add_procedure(new_routine)
        self.procedures.append(new_routine)
        return self[item]

    def set_precision(self, precision):
        assert isinstance(precision, int), "Precision argument in set_precision must be an integer"
        for routine in self.procedures:
            routine.set_precision(precision)
        self.precision = precision

    def main(self):
        for procedure in self.procedures:
            if isinstance(procedure, ModuleMain):
                return procedure
        return None


class ModuleMain(Procedure):
    def __init__(self, module):
        super(ModuleMain, self).__init__("main", module)
        self.module = module


class Routine(Procedure):
    def __init__(self, _name, module):
        super(Routine, self).__init__(_name, module)


class Function(Procedure):
    def __init__(self, _name, module):
        super(Function, self).__init__(_name, module)
        self.type = None


class Interface:
    def __init__(self, _name):
        self.name = _name
        self.module = None
        self.procedures = []

    def add_procedure(self, _procedure):
        assert isinstance(_procedure, Routine) or isinstance(_procedure, Function)

        self.procedures.append(_procedure)
        # Don't know it will be useful yet
        if not _procedure.interface:
            _procedure.interface = self
        elif isinstance(_procedure.interface, list):
            _procedure.interface.append(self)
        else:
            _procedure.interface = [_procedure.interface, self]




