from AutoRPE.UtilsRPE.BasicFunctions import close_brackets, remove_literals, literal_mask
from AutoRPE.UtilsRPE.BasicStructures import Function, Interface

import AutoRPE.UtilsRPE.Error as Error
import re


def calls_to_routines(string):
    pattern = "(^|[ \t]*)call +"
    matches = [(m.start(0), m.end(0)) for m in re.finditer(pattern, string, flags=re.I)]
    if matches:
        pieces = [string[start:].strip() for start, end in matches]
        return [close_brackets(piece) for piece in pieces if piece.count("(")]
    else:
        return []


def calls_to_functions(string, routine=None, vault=None):
    functions = calls_to_procedure(string, procedure_type=Function, vault=vault)
    interfaces = calls_to_procedure(string, procedure_type=Interface, vault=vault)
    # Assert that what was selected where functions
    to_return = functions + interfaces
    to_return = [fun for fun in to_return if is_call_to_function(fun, routine=routine, vault=vault)]
    return to_return


def calls_to_procedure(string, procedure_type, vault=None):

    # line without literals to avoid parenthesis inside strings and other ugly stuff
    no_literal_line = remove_literals(string)
    to_return = []
    if re.search(".*function .*", string, re.IGNORECASE):
        return to_return
    if re.search(".*subroutine.*", string, re.IGNORECASE):
        return to_return
    pattern = r"(\w+) *\("
    matches = [m for m in re.finditer(pattern, no_literal_line, flags=re.I)]
    if matches:
        if vault:
            for m in matches:
                fun_name = m.groups(0)[0]
                try:
                    fun = vault.get_procedure_by_name(fun_name, procedure_type=procedure_type)
                    # In case the function name corresponds to a interface, look for the corresponding function.
                    # if isinstance(fun, Interface):
                    #     function_call = close_brackets(string[m.start(1):])
                    #     arguments = find_call_arguments(function_call)
                    #     try:
                    #         fun = match_arguments_and_interface(arguments, fun, vault=vault)
                    #     # Ignore AssertionError and MemberNotFound exceptions.
                    #     except AssertionError:
                    #         fun = None
                    #     except MemberNotFound:
                    #         fun = None

                except Error.ProcedureNotFound:
                    fun = None
                # TODO: Think a way to avoid having nf90 exceptions inside routines
                if fun or (procedure_type == Function and fun_name.lower().find("nf90_") == 0):
                    function_call = close_brackets(string[m.start(1):])
                    to_return.append(function_call)
        else:
            for m in matches:
                function_call = close_brackets(string[m.start(1):])
                to_return.append(function_call)

    return to_return


def is_call_to_function(contents, routine=None, vault=None):
    """
    This function first does a first filter checking that the syntax is similar to a function call and afterwards
    uses one of the two possible approaches, depending on having a vault available or not.
    """
    pattern = r"[-+]* *(\w+) *\("
    m = re.match(pattern, contents)
    if m:
        if contents == close_brackets(contents):
            arguments = find_call_arguments(contents)
            # In case there are no arguments, for sure it is a function and not an array
            if len(arguments) == 0:
                return True
            # Otherwise, use one of the two approaches
            if vault is not None:
                return is_call_to_function_vault(contents, routine=routine, vault=vault)
            else:
                return is_call_to_function_no_vault(contents)
    return False


def find_called_routine(string):
    pattern = r"^[^\!']*[\t ]*call +([^\(]*)"
    match = re.search(pattern, string, flags=re.I)
    if match:
        return match.groups()[0].strip().lower()
    else:
        return None


def find_called_function(string):
    return string[:string.find("(")].strip()


# Description: Given a string with a comma separated arguments,
# extract the list of arguments getting rid of the parenthesis
# Input: String with subroutine declaration
# Output: List with arguments
def strip_call_arguments(_line):
    op = [i for i, x in enumerate(_line) if x == "("]
    cp = [i for i, x in enumerate(_line) if x == ")"]
    _line = _line[op[0] + 1:cp[-1]]
    op = [i for i, x in enumerate(_line) if x == "("]
    cp = [i for i, x in enumerate(_line) if x == ")"]
    commas = [i for i, x in enumerate(_line) if x == ","]
    splitted_line = [char for char in _line]
    for n in commas:
        if len([x for x in op if x < n]) == len([x for x in cp if x < n]):
            splitted_line[n] = "#"
    _line = "".join(splitted_line)
    return [_x.strip() for _x in _line.split("#")]


def find_call_arguments(string):
    n_arg, ints = find_argument_intervals(string)
    if n_arg == 0:
        return None
    if ints:
        arguments = [string[interval[0]:interval[1]].strip() for interval in ints]
        return [x for x in arguments if x]
    else:
        return None


def find_argument_intervals(string, _field=None):
    # Given a line with a call to a function, give the index intervals of the different arguments.

    # Mask to avoid scanning inside strings
    mask = literal_mask(string)

    # Find all opening parenthesis
    op = [i for i, x in enumerate(string) if x == "(" and not mask[i]]
    # Find all closing parenthesis
    cp = [i for i, x in enumerate(string) if x == ")" and not mask[i]]

    if not op and not cp:
        return 0, None

    # Keep only arguments part
    arguments_text = string[op[0] + 1:cp[-1]]
    # Find remaining parenthesis
    start = op[0] + 1
    end = cp[-1]
    op = [i for i, x in enumerate(arguments_text) if x == "("]
    cp = [i for i, x in enumerate(arguments_text) if x == ")"]
    osp = [i for i, x in enumerate(arguments_text) if x == "[" and not mask[i]]
    csp = [i for i, x in enumerate(arguments_text) if x == "]" and not mask[i]]

    # Find commas

    splitted_line = [c for c in arguments_text]

    commas = []
    mask = literal_mask(arguments_text)
    for i, c in enumerate(splitted_line):
        # Had to add this to avoid problems with commas inside strings"
        if c == ",":
            if not mask[i]:
                commas.append(i)

    # For existing commas:
    argument_separating_commas = []
    for n in commas:
        # If comma index n is outside parenthesis (number of openings equal number of closings) substitute
        if len([x for x in op if x < n]) == len([x for x in cp if x < n]):
            if len([x for x in osp if x < n]) == len([x for x in csp if x < n]):
                splitted_line[n] = "#"
                argument_separating_commas.append(n)

    # Displacing the argument_separating_commas to match the original line
    argument_separating_commas = [_x + start + 1 for _x in argument_separating_commas]
    # Adding first and last elements
    argument_separating_commas = [start] + argument_separating_commas + [end + 1]

    # Constructing intervals
    intervals = [(argument_separating_commas[i], argument_separating_commas[i + 1] - 1) for i in
                 range(len(argument_separating_commas) - 1)]
    if _field is not None:
        if len(intervals) <= _field:
            return len(intervals), None
        else:
            return len(intervals), intervals[_field]
    else:
        return len(intervals), intervals


def is_call_to_function_no_vault(contents):
    """
    This function tries to identify if a certain string is a call to a function or it's an array.
    We don't rely on nothing else than the string itself.
    """
    arguments = find_call_arguments(contents)
    for argument in arguments:
        if argument == ":":
            return False
    # TODO: This will wrongly identify arrays with integer incices as functions.
    return True


def is_call_to_function_vault(contents, routine, vault):
    from AutoRPE.UtilsRPE.Getter import get_variable
    """
    This function tries to identify if a certain string is a call to a function or it's an array.
    We rely on knowing in which routine we are and having a vault available.
    """
    pattern = r"[-+]* *(\w+) *\("
    function_name = re.match(pattern, contents).group(1)

    # Check if the name matches with the name of a variable in the actual routine
    if any([v for v in routine.variables if v.name == function_name]):
        return False
    # Search for a procedure with the same name
    try:
        fun = vault.get_procedure_by_name(function_name)
    except Error.ProcedureNotFound:
        fun = None
    if fun:
        return True

    # Search for a variable with the same name
    try:
        var = get_variable(function_name, routine, vault)
    except Error.VariableNotFound:
        var = None
    if var:
        if var.name == var.routine.name:
            print("Wait a minute!")
        return False
    # TODO: Here we are assuming that if it has not been identified neither as a function existing in the vault
    # TODO: nor a variable existing in the vault it corresponds to an external function. In case this assumption is not
    # TODO: True, this should be fixed.
    return True