from AutoRPE.UtilsRPE.BasicFunctions import literal_mask, close_brackets, is_all_inside_parenthesis
from AutoRPE.UtilsRPE.CallManager import is_call_to_function
import AutoRPE.UtilsRPE.RegexPattern as RegexPattern

import re


def remove_comments(string):
    # Trying to use a rarely used character ({) for the temporal replacement
    if string.count("!"):
        if string.count('$OMP'):
            return string
        # In the case where there's quotes in the text, we should ignore the exclamations inside
        if not string.count("'") and not string.count('"'):
            string = string[:string.find("!")]
        else:
            mask = literal_mask(string)
            # Looking for
            splitted_string = [c for c in string]
            splitted_string = ["{" if c == "!" and mask[i] else c for i, c in enumerate(splitted_string)]
            string = "".join(splitted_string)

            if string.find("!") >= 0:
                string = string[:string.find("!")]
            string = string.replace("{", "!")

    return string


def remove_if_condition(_line):
    if _line.replace(" ", "").lower().strip().find("if(") == 0:
        condition = close_brackets(_line)
        _line = _line.replace(condition, "")
    return _line


def remove_outer_brackets(string):
    string = string.strip()
    if is_all_inside_parenthesis(string):
        return string[1:-1].strip()
    else:
        return string


def remove_empty_lines(_lines):
    # Remove empty lines
    return [_l for _l in _lines if _l.strip()]


# Used in split_elements_in_operations
def remove_brackets(string):
    replacement_index = 0
    arrays = True
    replacement_list = []

    if string.strip()[0] == "[" and string.strip()[-1] == "]":
        string = string.strip()[1:-1]
    # Checks for all array pattern, and substitute with string
    while arrays:
        arrays = re.findall(RegexPattern.array, string)
        for array in arrays:
            string = string.replace(array, "REPLACEMENT" + str(replacement_index), 1)
            replacement_list.append(array)
            replacement_index += 1

        # Substitute structures with string
        structures = re.findall(r"\w+%\w+", string)
        for struct in structures:
            string = string.replace(struct, "REPLACEMENT" + str(replacement_index))
            replacement_list.append(struct)
            replacement_index += 1
            arrays = True
    functions = re.search(r"\w+\s*\(", string)
    if functions:
        matches = re.finditer(r"\w+\s*\(", string, flags=re.I)
        matches_start = [m.start(0) for m in matches]
        for start in matches_start:
            func = close_brackets(string[start:])
            string = string.replace(func, "REPLACEMENT" + str(replacement_index))
            replacement_list.append(func)
            replacement_index += 1

    # Now is safe to discard brackets
    string = string.replace("(", " ")
    string = string.replace(")", " ")

    # Substitute back the original structures
    for index, replacement in reversed(list(enumerate(replacement_list))):
        string = string.replace("REPLACEMENT" + str(index), replacement)

    return string


# Merge lines
# Definition:  given a file, puts back on the same line commands split with a &
def merge_lines(_lines):
    for _index, _line in enumerate(_lines):

        line = _lines[_index]
        if not line:
            continue
        if line[0] == "#":
            continue
        if line.count("&"):
            i = 1
            while line and line.strip()[-1] == "&":
                line1 = line
                line2 = _lines[_index + i]
                line = line1.replace("&", "").rstrip() + line2
                i += 1
            line = line.replace("&", "")
            # A commands to delete unnecessary spaces (gives some problems right now)
            _lines[_index] = line
            line = re.sub(", +", ", ", line)
            line = re.sub(":  +", ":  ", line)
            _lines[_index] = line
            for j in range(1, i):
                _lines[_index + j] = ""
    _lines = remove_empty_lines(_lines)
    return _lines


def clean_contents(contents, vault=None):
    if vault:
        intrinsics = vault.intrinsics
    else:
        intrinsics = ["reshape", "min", "max", "sum", "log", "maxval", "minval", "abs", "size", "nint"]

    if contents[0] == "-" or contents[0] == "+":
        contents = contents[1:]

    if re.match(r"\(/(.*)/\)", contents):
        contents = re.match(r"\(/(.*)/\)", contents).groups(1)[0]

    if contents[0] == "(":
        if "dummy%s" % contents == close_brackets("dummy%s" % contents):
            contents = contents[1:-1]

    if re.match(r"^([^\()]*) *\)+ *$", contents):
        contents = re.match(r"^([^\()]*) *\)+ *$", contents).groups(1)[0]

    if re.match(r".*\) *\(.*\)", contents):
        match = re.match(r"(.*\)) *\(.*\)", contents)
        contents = match.groups(0)[0]

    pattern = r"(\w+) *\("
    m = [_m for _m in re.finditer(pattern, contents)]
    if m:
        for mat in reversed(m):
            prefix = contents[:mat.start(1)]
            part_to_modify = contents[mat.start(1):]
            to_replace = close_brackets(part_to_modify)
            replacement = to_replace[:to_replace.find("(")]
            if replacement in intrinsics and is_call_to_function(contents):
                continue
            contents = "%s%s" % (prefix, part_to_modify.replace(to_replace, replacement))
    if contents.count("(/"):
        # Is temporal array
        contents = contents.replace("(/", "")
        contents = contents.replace("/)", "")
    return contents
