from AutoRPE.UtilsRPE.BasicFunctions import literal_mask, close_brackets
from AutoRPE.UtilsRPE.CurrentBlock import CurrentBlock
from AutoRPE.UtilsRPE.CallManager import find_argument_intervals, is_call_to_function
from AutoRPE.UtilsRPE.Inserter import replace_variable_exact_match
from AutoRPE.UtilsRPE.Splitter import split_elements_in_operations, split_elements_in_comparison
from AutoRPE.UtilsRPE.Getter import get_type_of_contents
from AutoRPE.UtilsRPE.SourceManager import list_of_intrinsics
from AutoRPE.UtilsRPE.NatureDeterminer import is_a_number


import re


# Checks that the argument is and rpe_var and returns the right fix
def fix_intrinsic_call_arguments(implement_rpe_file, argument, replacement, routine, cast=False):
    if argument.lower().count("already_fixed_"):
        return replacement

    argument_type = get_type_of_contents(argument, routine=routine, vault=implement_rpe_file.source_info)

    # No need to fix the argument
    if argument_type != 'rpe_var':
        return replacement

    if is_call_to_function(argument):
        return replace_variable_exact_match(argument, "real(" + argument + ")", replacement)
    else:
        if not cast:
            return replace_variable_exact_match(argument, argument + "%val", replacement)
        else:
            return replace_variable_exact_match(argument, "real(" + argument + ")", replacement)


# Fix all the intrinsics function that are not overloaded in the RPE library
def fix_intrinsic_calls(implement_rpe_file):
    current_block = CurrentBlock(implement_rpe_file.module.main())
    for _index, _line in enumerate(implement_rpe_file.lines):

        # Cycle through the matches, that can be more than one in a line
        replacements = []
        pieces_to_replace = []
        matches_start_list = []
        mask = literal_mask(_line)
        for intrinsic in list_of_intrinsics:

            # Search for the instrinsic
            fun = r"[ ,(+-=*](%s *)\(" % intrinsic

            matches = re.finditer(fun, _line, flags=re.I)
            matches_start = [m.start(1) for m in matches]

            if matches_start:
                for start in matches_start:
                    if not mask[start]:
                        matches_start_list.append(start)

        if not matches_start_list:
            current_block = current_block.in_which_block(_line, vault=implement_rpe_file.source_info)
            continue
        else:
            matches_start_list.sort(reverse=True)
            for start in matches_start_list:
                # Save the piece that need change
                pieces_to_replace.append(_line[start:].strip())

                # Current index
                i_match = len(pieces_to_replace) - 1
                # Remove initial minus sign (-)
                if pieces_to_replace[i_match][0] == "-":
                    pieces_to_replace[i_match] = pieces_to_replace[i_match][1:]
                # Remove brackets if they surrounds the argument
                if pieces_to_replace[i_match][0] == "(":
                    pieces_to_replace[i_match] = pieces_to_replace[i_match][1:]
                pieces_to_replace[i_match] = close_brackets(pieces_to_replace[i_match])
                _line = _line[:start] + _line[start:].replace(pieces_to_replace[i_match],
                                                              "ALREADY_FIXED_" + str(i_match))
                # Split the calls argument
                number_of_arguments, intervals = find_argument_intervals(pieces_to_replace[i_match])
                # Vector that will contain the changed pieces
                replacements.append(pieces_to_replace[i_match])

                for arg_id in range(number_of_arguments):
                    argument = pieces_to_replace[i_match][intervals[arg_id][0]:intervals[arg_id][1]]
                    if is_a_number(argument):
                        continue

                    # Check if the argument has to be split
                    elements = split_elements_in_comparison(argument)
                    if elements is None:
                        elements = split_elements_in_operations(argument)

                    if elements:
                        for element in elements:
                            replacements[i_match] = fix_intrinsic_call_arguments(implement_rpe_file,
                                                                                 element,
                                                                                 replacements[i_match],
                                                                                 current_block.routine)
                    else:
                        replacements[i_match] = fix_intrinsic_call_arguments(implement_rpe_file,
                                                                             argument,
                                                                             replacements[i_match],
                                                                             current_block.routine,
                                                                             cast=True)

        # for i_match, replacement in enumerate(replacements):
        for i_match, (piece_to_replace, replacement) in reversed(
                list(enumerate(zip(pieces_to_replace, replacements)))):
            _line = _line.replace("ALREADY_FIXED_" + str(i_match), piece_to_replace)
            if not pieces_to_replace == replacements:
                # Starting from the end, and substituting just first match we avoid re-substituting matches
                _line = _line.replace(piece_to_replace, replacement, 1)
        implement_rpe_file.lines[_index] = _line
