from AutoRPE.UtilsRPE.SourceManager import list_of_intrinsics
from AutoRPE.UtilsRPE.BasicFunctions import remove_literals
from AutoRPE.UtilsRPE.CallManager import is_call_to_function
import AutoRPE.UtilsRPE.RegexPattern as RegexPattern
import AutoRPE.UtilsRPE.VariablePrecision as VariablePrecision

import re


def is_start_function_statement(line):
    # Remove matches inside write(*,*) statements
    line = remove_literals(line)
    # Matches the word functions, excluding the end word
    start_function_pattern = r'function(?!(?<=\bend function)\s*)[^_]'
    is_start = re.search(start_function_pattern, line, re.MULTILINE | re.IGNORECASE)
    if is_start:
        return True
    return False


def is_end_function_statement(line):
    end_function_pattern = "^\s*end\s*function"
    is_end = re.match(end_function_pattern, line, re.I)
    if is_end:
        return True
    return False


# Check if a string is a number and returns the type
def is_a_number(string):
    if string.count("="):
        string = string.split("=")[1]

    # Check if is just white spaces
    if not string.strip():
        return False
    else:
        string = string.lower().strip()

    # Remove unary operator if present
    if string[0] == '-' or string[0] == '+':
        string = string[1:]

    name_of_intrinsic = has_intrinsics(string, True)
    if name_of_intrinsic:
        string = string.replace(name_of_intrinsic, "")
        string = string.strip()
        string = string[1:-1]
        string = string.strip()

    # Checks for number in exponential notation
    exp = re.match("^" + RegexPattern.number_exp_DPE_notation + "$", string)
    # Checks for number with precision specified
    real_with_prec = re.match(RegexPattern.real_number_with_precision, string)
    if exp or real_with_prec:
        # In case the number is a hardcoded real with a type specification,
        # check the lookup table to know to which type it corresponds and return it
        try:
            new_type = VariablePrecision.lookup_table[string.split("_")[1]]
        except IndexError:
            return VariablePrecision.wp
        return new_type

    # Check for pure integer and real numbers
    try:
        float(string)
        if string.isdigit():
            return "integer"
        else:
            return VariablePrecision.wp
    except ValueError:
        return False


def has_operations(string):
    operations = ["+", "-", "*", "/"]
    # Exclude key arguments
    # TODO: Deal with "list comprehension" i.e. : ' (/ (zshift + REAL(ji,wp), ji = 1, jpi_crs*jpj_crs) /)'
    # TODO: This approach does not work
    keyword_pattern = "^ *\w+ *=.*"
    if re.match(keyword_pattern, string, re.I):
        return False
    # Check if string is just white spaces
    if not string.strip():
        return False
    else:
        string = string.lower().replace(" ", "")

    if string.count("'") or string.count('"'):
        return False
    if is_call_to_function(string):
        return False
    # Remove the unary operators
    if string[0] == '-' or string[0] == '+':
        string = string[1:]

    # Replace char that can cause false positive
    string = string.replace("=-", 'EQUAL_MIN')
    string = string.replace("(/", "BEGIN_VEC(")
    string = string.replace("/)", ")")

    # Substitutes arrays and structures, then deletes useless brackets
    string = remove_arrays_and_structures(string)

    # Check for exponential, regex is needed to be sure is not just one of the variables that ends with 'e'
    exp_DPE = re.search(RegexPattern.number_exp_DPE_notation, string)
    if exp_DPE:
        string = string.replace(exp_DPE.group(), 'EXPONENTIAL_DPE')

    for op in operations:
        if op in string:
            return True
    return False


def has_intrinsics(string, return_intrinsic_name=False):
    string = string.lower()

    for key in list_of_key:
        if string.count(key):
            key_match = r",\s*[ ,(+-]%s *= *\w" % key
            string = re.sub(key_match, "", string)
    # We just need to check the assignation, not the variable name
    if string.count("="):
        string = string.split("=")[1].strip()

    # Split parenthesis opening
    if not string.count("("):
        return False

    for intrinsic in list_of_intrinsics:
        if re.search(r'\s*%s\s*\(' % intrinsic, string):
            if return_intrinsic_name:
                return intrinsic
            return True
    return False


# PRIVATE

# Substitutes arrays and structures, then deletes useless brackets
# Used in has_operations
def remove_arrays_and_structures(string, ret_rep_list=False):
    replacement_index = 0
    arrays = True
    replacement_list = []
    # Checks for all array pattern, and substitute with string
    while arrays:
        arrays = re.findall(RegexPattern.array, string)
        for array in arrays:
            string = string.replace(array, "REPLACEMENT" + str(replacement_index), 1)
            replacement_list.append(array)
            replacement_index += 1

        # Substitute structures with string
        structures = re.findall(r"\w+%\w+", string)
        for struct in structures:
            string = string.replace(struct, "REPLACEMENT" + str(replacement_index))
            replacement_list.append(struct)
            replacement_index += 1
            arrays = True
    if ret_rep_list:
        return string, replacement_list
    return string


list_of_key = ["dim", "mask", "kind", "back"]

