from AutoRPE.UtilsRPE.Getter import get_type_of_contents, get_variable
from AutoRPE.UtilsRPE.BasicFunctions import literal_mask
from AutoRPE.UtilsRPE.AttributeParser import attribute_split
from AutoRPE.UtilsRPE.Cleaner import remove_outer_brackets, remove_if_condition
from AutoRPE.UtilsRPE.CurrentBlock import CurrentBlock
import AutoRPE.UtilsRPE.VariablePrecision as VariablePrecision
import AutoRPE.UtilsRPE.RegexPattern as RegexPattern
import AutoRPE.UtilsRPE.Error as Error
import re


def process_pointers(vault):
    for var in vault.variables:
        if var.pointer:
            if var.pointer.count("type"):
                pointer_name = var.pointer.replace("type", "")
                pointer_name = remove_outer_brackets(pointer_name)
                search_array = vault.derived_types
            elif var.pointer.count("class"):
                # Same of type, but for compiler reason they have to be declared as classes
                pointer_name = var.pointer.replace("class", "")
                pointer_name = remove_outer_brackets(pointer_name)
                search_array = vault.derived_types
            elif var.type == "procedure":
                pointer_name = var.pointer
                search_array = vault.procedures
            else:
                # It is a pointer to real or a pointer to integer
                continue
            pointed_obj = [obj for obj in search_array if obj.name == pointer_name]
            if len(pointed_obj) > 1:
                raise Error.DuplicatedObject
            else:
                var.pointer = pointed_obj
            continue


def fix_pointer_assignations(implement_rpe_file):
    current_block = CurrentBlock(implement_rpe_file.module.main())

    for line_index, line in enumerate(implement_rpe_file.lines):
        pointer_assignation = re.search(RegexPattern.pointers_assignation, line)
        if not pointer_assignation:
            current_block = current_block.in_which_block(line, vault=implement_rpe_file.source_info)
        else:

            # Group 1 contains eventual pointer attributes
            pointer = pointer_assignation.group(2).strip()
            target = pointer_assignation.group(3).strip()
            pointer_type = get_type_of_contents(pointer, routine=current_block.routine, vault=implement_rpe_file.source_info)
            target_type = get_type_of_contents(target, routine=current_block.routine, vault=implement_rpe_file.source_info)
            if pointer_type in VariablePrecision.real_types and target_type == "rpe_var":
                implement_rpe_file.lines[line_index] = line.replace(target, "%s%%val" % target)


def get_pointer_target(line, routine, vault):
    # Ignore the if condition. This is not an ideal solution, but the easier for the moment
    line = remove_if_condition(line)
    if line.count("::"):
        line = line.split("::")[1]
    pointers = line.strip().split("=>")
    try:
        ptr = get_variable(pointers[0].strip(), routine=routine, vault=vault)
    except Error.VariableNotFound:
        # Is a pointer to routine
        if vault.get_procedure_by_name(pointers[1].strip(), module_name=routine.module.name):
            return [None, None]
        else:
            raise Error.VariableNotFound
    if pointers[1].lower().replace(" ", "") == "null()":
        return [None, None]
    else:
        # Is a pointer to procedure
        if ptr.type == 'procedure':
            try:
                from AutoRPE.UtilsRPE.BasicStructures import Function
                target = vault.get_procedure_by_name(pointers[1].strip(), procedure_type=Function, module_name=routine.module.name)
                # It is a pointer to function, i.e. a variable is associated with it
                return ptr, target
            except Error.ProcedureNotFound:
                # It is a pointer to subroutine
                return [None, None]
        else:
            target = get_variable(pointers[1].strip(), routine=routine, vault=vault)
    return ptr, target


def is_pointer(attributes, variables):
    # Description: Given a list of attributes, check if pointer is among them
    # Input: List of attributes
    # Output: boolean
    if isinstance(attributes, str):
        attributes = attribute_split(attributes)
    assert isinstance(attributes, list)
    if "pointer" in attributes:
        pointed_object = attributes[0]
        if pointed_object.count("procedure"):
            pointed_object = pointed_object.replace("procedure", "")
            pointed_object = remove_outer_brackets(pointed_object)
        return pointed_object
    if variables.count("=>"):
        pointed_object = variables.split("=>")[1]
        return pointed_object.strip()
    return False


def is_pointer_assignment(string):
    pattern = ".* => .*"
    mask = literal_mask(string)
    match = re.search(pattern, string, flags=re.I)
    if match and not mask[string.index("=>")]:
        return True
    return False
