from AutoRPE.UtilsRPE.BasicStructures import Interface, Module, Routine, Function
from AutoRPE.UtilsRPE.ObtainSourceFileInfo import ObtainSourcefileInfo
from AutoRPE.UtilsRPE.DerivedTypeDealer import DerivedType
from AutoRPE.UtilsRPE.Vault import Vault
from AutoRPE.UtilsRPE.ObtainSourceFileInfo import ReadSourceFile
from os.path import dirname, abspath, isfile, join
import AutoRPE.UtilsRPE.PointerDealer as PointerDealer
from __future__ import print_function
import pickle
import re


def parse_sources(path_to_model_sources, save_vault=False):
    # Find source-files in sources
    # (Using the sources that already modified replacing real declarations by type(rpe_var) )
    sources = [f for f in listdir_ord(path_to_model_sources) if
               isfile(join(path_to_model_sources, f)) and f.count(".f90")]
    # Initialize two empty dictionaries that will contain all the routines and all the functions
    object_oriented_dictionary = {"procedures": {}, "interfaces": {}, "variables": {}, "modules": {},
                                  "derivedtypes": {}}

    # Initialize an empty dictionary that will contain all the source-files.
    sourcefiles = {}
    vault = Vault()

    print("\n\nParsing sourcecode...\n")

    # Going over all the source-files loading the files,
    # parsing the routines and storing both things in dictionaries.
    for index, filename in enumerate(sources):
        module_name = filename.replace(".f90", "")
        sourcefiles[module_name] = ObtainSourcefileInfo(join(path_to_model_sources, filename))
        module = Module(module_name)
        f = sourcefiles[module_name]
        f.module = module
        f.find_routines()
        f.find_functions()
        f.find_module_variables()
        for procedure in f.objects:
            object_oriented_dictionary["procedures"][procedure.name] = procedure
            module.add_procedure(procedure)
            if procedure.name == "main":
                module.variables = procedure.variables
                for var in procedure.variables:
                    vault.variables.append(var)
        vault.add_module(module)
        for procedure in f.objects:
            vault.procedures.append(procedure)
            if isinstance(procedure, Routine):
                vault.routines.append(procedure)
                for var in procedure.variables:
                    vault.variables.append(var)
            if isinstance(procedure, Function):
                vault.functions.append(procedure)
                for var in procedure.variables:
                    vault.variables.append(var)

    # Going over all the source-files parsing the interfaces and storing them in dictionary.
    for index, filename in enumerate(sources):
        module_name = filename.replace(".f90", "")
        f = sourcefiles[module_name]
        f.find_interfaces(object_oriented_dictionary["procedures"])
        # Getting and saving modules used
        f.module = vault[module_name]
        f.find_used_modules(vault)
        for interface in f.objects:
            if isinstance(interface, Interface):
                object_oriented_dictionary["interfaces"][interface.name] = interface
                vault[module_name].procedures.append(interface)
                vault.interfaces.append(interface)
                vault.procedures.append(interface)

    # After having all the used parsed, propagate the inheritance, and add public var to available vars
    vault.expand_use_statement()

    # Going over all the source-files parsing the derived types and storing them in dictionary.
    for index, filename in enumerate(sources):
        module_name = filename.replace(".f90", "")
        f = sourcefiles[module_name]
        f.find_types()
        for derived_type in f.objects:
            if isinstance(derived_type, DerivedType):
                object_oriented_dictionary["derivedtypes"][derived_type.name] = derived_type
                vault[module_name].procedures.append(derived_type)
                vault.derived_types.append(derived_type)
                vault.procedures.append(derived_type)
                for var in derived_type.variables:
                    vault.variables.append(var)

    # Fill a dictionary to speed up search of variables
    vault.process_variables()
    # Now that vault.derived_types is full add extension to derived types if any
    vault.expand_derived_types()

    # Passing to pointed objects the actual function/subroutine they points to
    PointerDealer.process_pointers(vault)

    # # Find implicit function declarations
    # for filename in sourcefiles:
    #     module_name = filename.replace(".f90", "")
    #     f = sourcefiles[module_name]
    #     f.find_implicit_functions(vault=vault)
    #     for procedure in f.objects:
    #         if procedure not in vault[module_name].procedures:
    #             vault[module_name].procedures.append(procedure)
    #             vault.procedures.append(procedure)

    print("number of functions " + str(len(vault.functions)) + "\n" +
          "number or routines " + str(len(vault.routines)) + "\n" +
          "number or interfaces " + str(len(vault.interfaces)) + "\n" +
          "number or main " + str(len(vault.modules)) + "\n" +
          "number or procedures " + str(len(vault.procedures)) + "\n" +
          "number of variables " + str(len(vault.variables)) + "\n" +
          "number or derived_types " + str(len(vault.derived_types)) + "\n")

    if save_vault:
        # This workaround is needed to allow pickle to write the file, don't know yet about its risks
        import sys
        sys.setrecursionlimit(2000)
        pickle.dump(vault, open("vault.pkl", "wb"))
    return vault


def preprocess_sources(path_to_input_sources, path_to_output_sources, blacklist=[]):
    from AutoRPE.UtilsRPE.ImplementRPE import ImplementRPE
    # Read all files sources excluding hidden files ( like vim swp files)
    all_sources = [f for f in listdir_ord(path_to_input_sources) if
                   isfile(join(path_to_input_sources, f)) and f.count(".f90") and f[0][0] != "."]
    total = len(all_sources)
    if len(all_sources):
        # Load and format files
        for index, filename in enumerate(all_sources):
            f = ImplementRPE(join(path_to_input_sources, filename), None)
            # Remove comments, empty lines and old style declarations
            if filename in blacklist:
                print("\rFormatting file %i/%i  %20s" % (index + 1, total, filename), end="")
                f.pre_process()
            # Additionally reals are replaced with rpe_var type
            else:
                print("\rFormatting file and replacing reals %i/%i  %20s" % (index + 1, total, filename), end="")
                f.pre_process(replace_reals=True)
            f.save_modified_file(join(path_to_output_sources, filename))
    else:
        # Sources is an empty folder
        raise ValueError("No files found in %s folder", path_to_input_sources)


def postprocess_sources(path_to_input_sources, path_to_output_sources, vault):
    from AutoRPE.UtilsRPE.ImplementRPE import ImplementRPE

    # List files in source sexcluding hidden files ( like vim swp files)
    all_sources = [f for f in listdir_ord(path_to_input_sources) if
                   isfile(join(path_to_input_sources, f)) and f.count(".f90")and f[0][0] != "."]
    total = len(all_sources)
    for index, filename in enumerate(all_sources):
        f = ImplementRPE(join(path_to_input_sources, filename), vault)
        print("\rFixing issues %i/%i  %20s" % (index + 1, total, filename), end="")
        f.post_process()
        f.save_modified_file(join(path_to_output_sources, filename))


# Get all files contained in a source path (recursive)
def expand_sources(source_path):
    from os import walk
    from os.path import join
    import re
    pattern = ".*\.[fFh90]"
    files = [join(dp, f) for dp, dn, fn in walk(source_path) for f in fn]
    files = [f for f in files if re.search(pattern, f)]
    # Sorting the list to solve the random-like behaviour of walk
    files = list(sorted(files))
    return files


# Gets the filepath of the corresponding module
def match_module_and_filename(module, filenames):
    import re
    pattern = ".*/%s\.[fFh90]" % module
    matches = [fn for fn in filenames if re.match(pattern, fn)]
    return matches


def load_variable_list(filename):
    # Load list of variables
    from numpy import loadtxt
    variables = loadtxt(filename, dtype=int).tolist()
    if not isinstance(variables, list):
        variables = [variables]
    return variables


def load_source_files(path):
    filelist = [ReadSourceFile(join(path, filename)) for filename in listdir_ord(path) if filename.count(".f90")]
    filedict = {}
    for f in filelist:
        filedict[f.filename] = f
    return filedict


def load_file(filename):
    with open(filename, "r", encoding="latin-1") as f:
        # Read, strip and keep only lines with content
        _lines = [_l.strip() for _l in f if _l.strip()]
    return _lines


def listdir_ord(path):
    from os import listdir
    file_list = listdir(path)
    file_list.sort()
    return file_list


# Hardcoded list, this should be solved
list_of_intrinsics = ["max", "maxloc", "maxval", "min", "minloc", "minval", "epsilon",
                      "isnan", "count", "abs", "mod", "reshape",
                      "atanh", "sqrt", "log",
                      "sum",
                      "erfc",
                      "dsqrt", "cmplx", "transfer"]


def load_intrinsics():
    package_directory = dirname(abspath(__file__))
    # Hardcoded path to file.
    with open(package_directory + "/../AdditionalFiles/fortran_intrinsics") as f:
        return [re.search(r"(\w+)", l).groups(0)[0].lower() for l in f]


def load_vault(filename="vault.pkl"):
    try:
        return pickle.load(open(filename, "rb"))
    except IOError:
        return None


def load_list_of_files_to_keep_unmodified(filename=None):
    if filename is None:
        package_directory = dirname(abspath(__file__))
        filename = package_directory + "/../AdditionalFiles/list_of_files_to_keep_unmodified.txt"
    # Hardcoded path to file.
    if isfile(filename):
        print("Loading list of exceptions from %s\n\n" % filename)
        with open(filename) as f:
            return [l.strip() for l in f if l.strip() and l.strip()[0] != "#"]
    else:
        return None


def store_variable_list(list_of_variables, filename, vault):
    # Save list of variables into a txt file
    # Sort and keep only uniq values
    list_of_variables = list(sorted(set(list_of_variables)))
    line_format = "%5i    # %20s %25s %20s\n"
    with open(filename, "w") as output_file:
        # Add header
        output_file.write("#%5s     %20s %25s %20s\n%s\n" % ("ID", "variable", "routine", "module", 100 * "#"))
        # Add variables
        for var_id in list_of_variables:
            v = vault.get_variable_by_id(var_id)
            output_file.write(line_format % (v.id, v.name, v.routine.name, v.routine.module.name))



def store_lines_into_file(_lines, filename):
    to_write = "\n".join(_lines).strip()
    with open(filename, "w") as f:
        f.write(to_write)
