from AutoRPE.UtilsRPE.BasicFunctions import is_all_inside_parenthesis, close_brackets
from AutoRPE.UtilsRPE.Cleaner import remove_brackets, remove_outer_brackets
from AutoRPE.UtilsRPE.Finder import find_var_elsewhere
from AutoRPE.UtilsRPE.CallManager import strip_call_arguments, calls_to_functions
from AutoRPE.UtilsRPE.NatureDeterminer import has_operations
import AutoRPE.UtilsRPE.RegexPattern as RegexPattern
import AutoRPE.UtilsRPE.Error as Error
import re


def split_hardcoded_array(string):
    match = re.search(RegexPattern.hardcoded_array, string)
    if match:
        # Remove the parenthesis
        string = string.replace(match.group(1), "")
        string = string.replace(match.group(2), "")
    else:
        raise Error.IsNotAnArray
    return strip_call_arguments("(" + remove_outer_brackets(string) + ")")


def split_elements_in_operations(string, routine=None, vault=None):
    if not has_operations(string):
        return None
    operations = ["+", "-", "*", "/"]
    exponentials_DPE = re.findall(RegexPattern.number_exp_DPE_notation, string, re.I)
    pattern_pow = r'\*\*\d+\.?'
    # pows = re.findall(pattern_pow, string, re.I)
    # for pow in pows:
    #     string = string.replace(pow, "")
    string.replace("(/", "BEGIN_VECTOR")
    string.replace("/)", "END_VECTOR")
    # Substitute exponential with string
    for index, exponential_DPE in enumerate(exponentials_DPE):
        string = string.replace(exponential_DPE, "EXPONENTIAL_DPE" + str(index))

    string = remove_brackets(string)

    split_elements = split_elements_by_symbols(string, operations, routine=routine, vault=vault)

    for element in split_elements:
        element.replace("BEGIN_VECTOR", "(/")
        element.replace("END_VECTOR", "/)")

    # Substitute back expontenials.
    for index, exponential_DPE in enumerate(exponentials_DPE):
        sub = "EXPONENTIAL_DPE" + str(index)
        for element in split_elements:
            if element.count(sub):
                split_elements[split_elements.index(element)] = element.replace(sub, exponential_DPE)
    return split_elements


def split_elements_in_comparison(string):
    comparison_operators = [">=", "<=", "==", "/=", ">", "<", ".or.", ".and."]
    elements_in_comparison = []

    string = remove_brackets(string.lower())

    for operators in comparison_operators:
        if string.count(operators):
            for el in string.split(operators):
                elements_in_comparison.append(el.strip())
            break
    if elements_in_comparison:
        return elements_in_comparison
    else:
        return None


def split_by_function(variables, fun, files=None):
    pattern = r"[^\w]%s[^\w]"

    vars_true = []
    vars_false = []

    for var in variables:
        appears_in = find_var_elsewhere(var, files)
        found = False
        for f in appears_in:
            appears_in_lines = [line for line in f.lines if
                                re.search(pattern % var.name, line) and not line.count("::")]
            found_array = [True if fun(line) else False for line in appears_in_lines]
            if any(found_array):
                found = True
        if found:
            vars_true.append(var)
        else:
            vars_false.append(var)
    return vars_true, vars_false


def split_elements_by_symbols(string, symbols, routine=None, vault=None):
    functions = calls_to_functions(string, routine=routine, vault=vault)
    dictionary_of_replacements = {}
    if functions:
        for index, function in enumerate(functions):
            dictionary_of_replacements["REPLACEMENT%i" % index] = function
            string = string.replace(function, "REPLACEMENT%i" % index)

    listed_text = [c for c in string]
    parenthesis_counter = 0
    for index, c in enumerate(listed_text):
        if c == "(":
            parenthesis_counter += 1
        elif c == ")":
            parenthesis_counter -= 1
        elif c in symbols and parenthesis_counter == 0:
            listed_text[index] = "#"
    string = "".join(listed_text)
    arguments = [x.strip() for x in string.split("#") if x.strip()]
    for index, argument in enumerate(arguments):
        for key, function in dictionary_of_replacements.items():
            if key in argument:
                arguments[index] = arguments[index].replace(key, function)

    return arguments


def split_vars_by_allocatable(variables):
    allocatables = [var for var in variables if var.type == "rpe_var" and var.allocatable]
    non_allocatables = [var for var in variables if var.type == "rpe_var" and not var.allocatable]
    return allocatables, non_allocatables


def split_by_appearance_in_namelist(variables, files=None):
    return split_by_function(variables, is_namelist_line, files=files)


def split_vars_by_main_or_routine(variables):
    in_main = [var for var in variables if var.routine.name == "main"]
    not_in_main = [var for var in variables if var.routine.name != "main"]

    return in_main, not_in_main


def split_read_arguments(string):
    arguments = []

    # First check for structure
    match_struct = re.findall(RegexPattern.array_with_array_member, string, re.I)
    for struct in match_struct:
        arguments.append(struct)
        string = string.replace(struct, "")
        if not string:
            return arguments
    # Now for vectors
    match_vec = re.findall(RegexPattern.array_with_member, string, re.I)
    for vec in match_vec:
        arguments.append(RegexPattern.array)
        string = string.replace(RegexPattern.array, "")
        if not string:
            return arguments

    # now that all vectors are pruned parenthesis can be deleted
    string = string.replace("(", "")
    string = string.replace(")", "")
    # Check for lists
    if string.count("="):
        # remove index of lists, that are integers
        indexes = re.findall(r"\w+=\d+", string)
        for index in indexes:
            string = string.replace(index, "")
    # The rest is just scalar
    for scalar in string.split(","):
        # add just if is not empty
        if not scalar:
            continue
        arguments.append(scalar)
    return arguments


def split_line_container_and_arguments(string):
    # Given a line containing a call to a write function, it returns the list of the arguments and the
    # remaining of the line without the arguments.

    # Locate start of write call
    loc = re.search(r"(\W|^)(write)\W", string, re.I).start(2)
    # Save the part of the string that goes before the call if any
    pre_text = string[:loc]
    # Keep the rest of the string
    post_text = string[loc:]
    # Temporarily replace write statement with all the parameters
    write_statement = close_brackets(post_text)
    line = post_text.replace(write_statement, "write()")

    # Find match (?)
    pattern = r"^[^\!]* *write\([^()]*\) (.*)"
    matches = re.search(pattern, line, re.IGNORECASE)
    to_replace = matches.groups(0)[0]

    # Prepare line container
    if to_replace.strip():
        line_container = line.replace(to_replace, "###TOREPLACE###")
    else:
        line_container = line
        return line_container, []

    arguments = to_replace.strip()

    if not arguments:
        return line_container, []

    # Fix array constructors
    if arguments.find("(/") == 0:
        arguments = arguments[2:-2].strip()
        line_container = line_container.replace("###TOREPLACE###", "(/ ###TOREPLACE### /)")

    if is_all_inside_parenthesis(arguments):
        arguments = arguments[1:-1].strip()
        line_container = line_container.replace("###TOREPLACE###", "( ###TOREPLACE### )")

    arguments = split_write_arguments(arguments)

    line_container = pre_text + line_container.replace("write()", write_statement)
    return line_container, arguments


def split_write_arguments(string):
    # Given a line containing a call to a read function, it returns the list of the arguments
    # pattern = "^[^\!]* +read\([^()]*\) *(.*)"
    # matches = re.search(pattern, string, re.IGNORECASE)
    arguments = string

    scan_quotation = [arguments[:i].count("'") for i, c in enumerate(arguments)]
    scan_quotation2 = [arguments[:i].count("\"") for i, c in enumerate(arguments)]
    scan_op_brackets = [arguments[:i].count("(") for i, c in enumerate(arguments)]
    scan_cl_brackets = [arguments[:i].count(")") for i, c in enumerate(arguments)]
    scan_brackets = [i - j for i, j in zip(scan_op_brackets, scan_cl_brackets)]
    split_text = [c for c in arguments]
    split_text = [
        "#" if c == "," and (scan_quotation[i] % 2 == 0 and scan_quotation2[i] % 2 == 0) and not scan_brackets[i] else c
        for i, c in enumerate(split_text)]
    arguments = "".join(split_text)
    arguments = [a for a in arguments.split("#") if a.strip()]
    return arguments


def split_semicolon(string):
    # Trying to use a rarely used character ({) for the temporal replacement
    if string.count(";"):
        # In the case where there's quotes in the text, we should ignore the exclamations inside
        if not string.count("'") and not string.count("\""):
            return string.split(";")
        else:
            # Looking for
            scan_quotation = [string[:i].count("'") for i, c in enumerate(string)]
            scan_quotation2 = [string[:i].count("\"") for i, c in enumerate(string)]
            splitted_text = [c for c in string]
            splitted_text = ["{" if c == ";" and (scan_quotation[i] % 2 == 1 or scan_quotation2[i] % 2 == 1) else c for
                             i, c in
                             enumerate(splitted_text)]
            string = "".join(splitted_text)
            return string.split(";")

    raise AssertionError("string must contain semi-colon")


## PRIVATE FUNCTION
def is_namelist_line(line):
    return line.lower().strip().find("namelist") == 0

