from AutoRPE.UtilsRPE.BasicStructures import Variable
from AutoRPE.UtilsRPE.BasicFunctions import close_brackets
from AutoRPE.UtilsRPE.CallManager import find_argument_intervals


def use_temporal_variable(sourcefile, line_index, argument, argument_index, routine=None, dummy=None, counter=None):
    tmp_var = "tmp_var_%i" % counter.count
    var_object = Variable(routine, tmp_var, "rpe_var", dummy.dimension)
    routine.add_variable(var_object)
    counter.up()

    temporal_call = sourcefile.lines[line_index]
    if temporal_call.lower().strip().find("if(") == 0:
        condition = close_brackets(temporal_call)
    else:
        condition = ""

    # Temporal variable declaration
    temporal_variable_declaration = temporal_variable_allocation_line(condition, tmp_var, argument, dummy.dimension)

    # Allocation
    allocation = temporal_variable_allocation_line(condition, tmp_var, argument, dummy.dimension)

    # Deallocation
    if not dummy.dimension:
        deallocation = condition + "deallocate(%s)" % tmp_var
    else:
        deallocation = ""

    # Pre assignation
    preassignation = condition + "%s = %s" % (tmp_var, argument)

    # Post assignation
    postassignation = condition + "%s = %s" % (argument, tmp_var)

    # Insert declaration line
    declaration_line = line_index
    for i in reversed(range(line_index)):
        if sourcefile.lines[i].count("::"):
            declaration_line = i
            break
    sourcefile.lines.insert(declaration_line + 1, temporal_variable_declaration)

    line_index += 1
    # Insert allocation
    sourcefile.lines.insert(line_index, allocation)
    # Insert preassignation
    sourcefile.lines.insert(line_index + 1, preassignation)

    line_index += 2

    # Replace call
    call = replace_argument(temporal_call, argument_index, tmp_var)

    sourcefile.lines[line_index] = call

    sourcefile.lines.insert(line_index + 1, postassignation)

    # Insert deallocation
    sourcefile.lines.insert(line_index + 2, deallocation)


## PRIVATE
def temporal_variable_allocation_line(condition, temporal_variable_name, original_variable_name, dimension):
    if not dimension:
        assert isinstance(dimension, int)
        if dimension == 1:
            return condition + " allocate(%s( size(%s) ) )" % (temporal_variable_name, original_variable_name)
        elif dimension < 0:
            raise AssertionError("Dimension can not be negative!")
        else:
            addition = "size(%s,1)" % original_variable_name
            for i in range(2, dimension + 1):
                addition += ", size(%s,%i)" % (original_variable_name, i)
            return condition + " allocate(%s( %s ) )" % (temporal_variable_name, addition)
    else:
        return ""


def replace_argument(string, _field, new_argument):
    # Keep the condition if exists
    if string.lower().strip().find("if(") == 0:
        condition = close_brackets(string)
        has_condition = True
        string = string.replace(condition, "%REPLACEMENT%")
    else:
        has_condition = False

    num_arguments, intervals = find_argument_intervals(string)
    interval = intervals[_field]
    start = interval[0]
    end = interval[1]
    first_segment = string[:start]
    last_segment = string[end:]
    string = "%s %s %s" % (first_segment, new_argument, last_segment)
    if has_condition:
        string = string.replace("%REPLACEMENT%", condition)
    return string
