def checkpoint(queue, iteration, save=True, filename="checkpoint.pkl"):
    if save is True:
        print("Saving checkpoint!", end="\r")
        import pickle
        import sys
        sys.setrecursionlimit(2000)

        all_jobs = queue.all() + queue.disinherited

        for job in all_jobs:
            if job.assertion_job is not None:
                all_jobs.append(job.assertion_job)
        # Save the communicator
        communicator = all_jobs[0].communicator
        # vault = all_jobs[0].vault
        for job in all_jobs:
            job.communicator = None
            # vault = None

        to_save = [queue, iteration]
        # Dump object
        pickle.dump(to_save, open(filename, "wb"))

        # Restore the communicator
        for job in all_jobs:
            job.communicator = communicator
        print("Checkpoint saved at %s!" % filename)


def recover_checkpoint(communicator, vault=None, filename="checkpoint.pkl"):
    import pickle as pkl
    queue, iteration = pkl.load(open(filename, "rb"))
    all_jobs = queue.all() + queue.disinherited
    for job in all_jobs:
        job.communicator = communicator
        if vault:
            job.vault = vault
    job_with_no_parent = [job for job in all_jobs if job.parent is None and job.level == 0]
    grandfather = job_with_no_parent[0]
    return queue, grandfather, iteration


def saving_state(_queue, old_hash=None, filename="state.pkl"):
    import pickle as pkl
    import hashlib
    possible_states = ["PENDING", "RUNNING", "ASSERTION_PENDING", "SUCCESS", "FAILED", "SUSPENDED"]
    log = {}
    for state in possible_states:
        log[state] = []

    for job in _queue.all():
        log[job.status].append(job.identifier())

    p = pkl.dumps(log)
    new_hash = hashlib.md5(p).hexdigest()
    if new_hash != old_hash:
        with open(filename, "wb") as output_log:
            pkl.dump(log, output_log, protocol=2)
    return new_hash



